(function(global){
    var app;

    function getDatabase(){
        return new MemoryStorage();
        if((window.cordova && (cordova.platformId === "ios")) || (navigator.userAgent.match(/(iPod|iPhone|iPad)/))) {
            return new WebSQLStorage();
        }

        if(window.indexedDB || window.mozIndexedDB || window.webkitIndexedDB || window.msIndexedDB) {
            return new IndexedDBStorage();
        } else {
            return new WebSQLStorage();
        }
    }

    function App(onReady){
        app = this;

        this.appBody = document.getElementById("appBody");

        if(!this.isCompatible()) {
            this.renderNoCompatiblePage();
            return;
        }

        this.renderBluePage();

        app.initSmartBanner();

        app.parseSearch();

        this.player = getPlayer();
        this.storage = getStorage();
        this.user = new User();

        global.db = getDatabase();

        this.synchronization = new Synchronization();

        if(localStorage.categoriesUnlocked) {
            app.categoriesUnlocked = JSON.parse(localStorage.categoriesUnlocked);
        } else {
            app.categoriesUnlocked = [];
        }

        if(localStorage.notPassedCategories) {
            app.notPassedCategories = JSON.parse(localStorage.notPassedCategories);
        } else {
            app.notPassedCategories = [];
        }

        window.addEventListener('popstate', function(e){
            if(e.state) {
                var scope = e.state.scope,
                    page = e.state.page;

                if(scope=="user") {
                    scope = app.user;
                } else if(scope=="game"){
                    scope = app.game;
                } else {
                    scope = app;
                }

                if(app.appBody.hasClass("playing")) {
                    app.game && app.game.timerWidget.pauseOn();
                    dialog(
                        _("The game isn’t over"),
                        _("If you quit now, you will lose your points."), [
                        {
                            value: _("Play "),
                            callback: function(){
                                window.history.pushState({
                                    page: "renderStartPage",
                                    scope: "game"
                                }, "Yes No Game");
                                app.game.timerWidget.pauseOff();
                            }
                        }, {
                            value: _("Quit"),
                            callback: function(){
                                app.appBody.removeClass("playing");
                                app.doNotTrackHistory = true;
                                scope[page]();
                            }
                        }
                    ]);
                } else {
                    app.doNotTrackHistory = true;
                    scope[page]();
                }
            }
        }, false);
    }

    App.prototype = {
        parseSearch: function (){
            this.params = {};

            location.search.replace("?", "").split("&").forEach(function(param){
                var a = param.split("=");
                this.params[a[0]] = a[1];
            }, this);
        },
        initSmartBanner: function(){
            if(!window.cordova) {
                $.smartbanner({
                    price: 'FREE', // Price of the app
                    inAppStore: 'On the App Store', // Text of price for iOS
                    inGooglePlay: 'In Google Play', // Text of price for Android
                    // inWindowsStore: 'In the Windows Store', // Text of price for Windows
                    button: 'VIEW', // Text for the install button
                    daysHidden: 15, // Duration to hide the banner after being closed (0 = always show banner)
                    daysReminder: 90, // Duration to hide the banner after "VIEW" is clicked *separate from when the close button is clicked* (0 = always show banner)
                    hideOnInstall: true, // Hide the banner after "VIEW" is clicked.
                    iOSUniversalApp: true, // If the iOS App is a universal app for both iPad and iPhone, display Smart Banner to iPad users, too.
                    layer: true
                });
            }
        },
        start: function(){
            var that = this;
            app.renderBluePage();
            app.checkOnline(true,
                this.startOnline.bind(this),
                this.startOffline.bind(this)
            );

            app.loadReviewQuestions();

            app.startTimestamp = +new Date();
        },
        startOnline: function(){
            if("auth" in app.params && app.params.auth=="sso") {
                app.ssoToken = app.params.ID;

                app.user.signInWithSSO(app.ssoToken, function(){
                    delete app.params.auth;
                    app.start();
                }, function(){
                    console.log(arguments);
                    delete app.params.auth;
                    app.start();
                });
            } else if(app.synchronization.currentVersion === 0) {
                if(app.isIOS() && !window.cordova) {
                    setTimeout(function(){
                        app.synchronization.syncInitial();
                    }, 3000);
                } else {
                    app.synchronization.syncInitial();
                }

            } else {
                this.synchronization.checkSyncAvailability(function(available){
                    if(available) {
                        app.hideCordovaSplashScreen();

                        dialog(_("Update available"), _("Do you want to update now?"), [{
                            value: _("Yes"),
                            callback: function(){
                                app.synchronization.syncInitial();
                            }
                        }, {
                            value: _("Skip"),
                            callback: app.startApplication.bind(app)
                        }]);
                    } else {
                        app.startApplication();
                    }
                });
            }
        },
        startOffline: function(){
            if(app.synchronization.currentVersion === 0) {
                app.connecting(function(){
                    app.startOnline();
                });
            } else {
                app.updateCategoriesWithSound();

                app.user.usePrevTimeUser(
                    function(){ // success
                        app.renderMenuPage();
                    },
                    function(){ // fail
                        app.user.useAnonymousUserOffline(
                            function(){
                                app.clearReviewQuestions();
                                app.renderMenuPage();
                            }
                        )
                    }
                );

                window.initFacebookSDK();
                window.gaInit();
            }
        },
        startApplication: function(){
            app.updateCategoriesWithSound();
            app.updateCountriesWithSchools();

            window.initFacebookSDK();
            window.gaInit();

            if("invite" in app.params) {
                quizAPI.callURL("invitedUserCome", {
                    code: app.params.invite
                });
            }

            app.hideCordovaSplashScreen();
            app.user.usePrevTimeUser(
                function(){ // success
                    app.synchronization.sendLocalGamesToServer();
                    if(app.isAnonymousUser()){
                        if("fblogin" in app.params){
                            if("FB" in window) {
                                app.user.signInWithFacebook();
                            } else {
                                setTimeout(function(){
                                    app.clearReviewQuestions();
                                    app.startApplication();
                                }, 500);
                            }
                        } else {
                            app.clearReviewQuestions();
                            app.user.renderStartPage();
                        }
                    } else {
                        app.renderMenuPage();
                    }
                },
                function(){ // fail
                    if("fblogin" in app.params){
                        if("FB" in window) {
                            app.user.signInWithFacebook();
                        } else {
                            setTimeout(function(){
                                app.startApplication();
                            }, 500);
                        }
                    } else {
                        app.clearReviewQuestions();
                        app.user.renderStartPage();
                    }
                }
            );
        },
        checkOnline: function(firstTime, onlineCallback, offlineCallback){
            var xhr = new XMLHttpRequest(),
                file = settings.apiURL + "ping/";

            xhr.open('HEAD', file + "?ts=" + (+new Date()), true);
            xhr.timeout = 3000;

            xhr.onload = function(e){
                if (xhr.status >= 200 && xhr.status < 304) {
                    if(!firstTime && !app.onLine) {
                        app.goOnline();
                    }
                    app.onLine = true;
                    onlineCallback && onlineCallback();
                } else {
                    if(app.onLine) {
                        app.goOffline();
                    }
                    app.onLine = false;
                    offlineCallback && offlineCallback();
                }
            }

            xhr.onerror = function(){
                if(app.onLine) {
                    app.goOffline();
                }
                app.onLine = false;
                offlineCallback && offlineCallback();
            }

            xhr.ontimeout = function(){
                if(app.onLine) {
                    app.goOffline();
                }
                app.onLine = false;
                offlineCallback && offlineCallback();
            }

            xhr.send();
        },
        goOnline: function(){
            this.onLine = true;
            app.synchronization.sendLocalGamesToServer();
        },
        goOffline: function(){
            this.onLine = false;
            this.checkOnLineTimeout = 1000;
            this.reconnectionTryTime = 0;
            this.reconnection();
        },
        reconnection: function(){
            this.reconnectionTryTime++;
            if(this.reconnectionTryTime == 5) { this.checkOnLineTimeout = 30000; }
            setTimeout(function(){
                app.checkOnline(false,
                    function(){
                        app.goOnline();
                    },
                    app.reconnection.bind(app)
                )
            }, this.checkOnLineTimeout);
        },
        updateCategoriesWithSound: function(){
            if(localStorage.categoriesWithSound) {
                app.categoriesWithSound = JSON.parse(localStorage.categoriesWithSound);
            } else {
                app.categoriesWithSound = [];
            }
        },
        updateCountriesWithSchools: function(){
            if(localStorage.countriesWithSchools) {
                app.countriesWithSchools = JSON.parse(localStorage.countriesWithSchools);
            }

            quizAPI.callURL("countriesWithSchools", {}, function(data){
                app.countriesWithSchools = data.countries;
                localStorage.countriesWithSchools = JSON.stringify(data.countries);
            });
        },
        isSchoolRankingEnabled: function(){
            return app.countriesWithSchools && app.countriesWithSchools.indexOf(app.user.user.countryCode) !== -1;
        },
        addPage: function(pageFunc){
            if(!this.prevPages) {
                this.prevPages = [];
            }

            this.prevPages.push(pageFunc);
        },
        goBack: function(){
            var prevPage = this.prevPages[this.prevPages.length-2];

            if(prevPage) {
                this.prevPages.splice(this.prevPages.length-2);
                app.categoriesPageWasRendered = false;
                app.joinPageWasRendered = false;
                app.appBody.removeClass("playing");
                prevPage();
            }
        },
        connecting: function(callback){
            this.renderConnectingPage();

            app.checkOnline(false, callback, function(){
                app.connecting(callback);
            });
        },
        startGame: function(category){
            app.game = new Game(category);
        },
        syncAndStartGame: function(category, skipSyncVersion){
            var  checkOnlineInterval, onlineCallback, offlineCallback, syncCategory;
            db.syncStart();

            app.startGame(category);
            loading.start();

            onlineCallback = function(){},
            offlineCallback = function(){
                clearInterval(checkInternetInterval);

                quizAPI.stopAllRequests();
                syncCategory.stopSynchronization();
                loading.stop();

                app.showCategoriesPage();
                dialog("Oh no!", "We can't connect to Yes No Game", [{
                    value: "OK"
                }]);
            }

            checkInternetInterval = setInterval(function(){
                app.checkOnline(false, onlineCallback, offlineCallback);
            }, 10000);

            syncCategory = app.synchronization.syncCategory(category.categoryID, function(){
                app.game.refreshQuestions(function(){
                    clearInterval(checkInternetInterval);
                    loading.stop();
                    db.syncFinish();
                    app.storage.save();
                });
            }, skipSyncVersion);
        },
        loadCategories: function(callback){
            db.categoryStore.filter("enabled", true, function(categories){
                if(categories.length === 0) {
                    app.synchronization.reset();
                    app.synchronization.syncInitial();
                    return;
                }

                app.categories = categories.sort(function(a, b){
                    if(a.indexNumber<b.indexNumber) return -1;
                    if(a.indexNumber>b.indexNumber) return 1;
                    return 0;
                });

                if(app.categoriesUnlocked.length===0){
                    app.categoriesUnlocked = [app.categories[0].categoryID];
                }

                callback();
            });
        },
        showCategoriesPage: function(){
            if(app.onLine) {
                if((+new Date() - app.startTimestamp) > (4 * 60 * 60 * 1000)){
                    this.synchronization.checkSyncAvailability(function(available){
                        if(available) {
                            dialog(_("Update available"), _("Do you want to update now?"), [{
                                value: _("Yes"),
                                callback: function(){
                                    app.synchronization.syncInitial(function(){
                                        app.startTimestamp = +new Date();
                                        app.loadAndShowCategoriesPage();
                                    });
                                }
                            }, {
                                value: _("Skip"),
                                callback: function(){
                                    app.startTimestamp = +new Date();
                                    app.loadAndShowCategoriesPage();
                                }
                            }]);
                        } else {
                            app.startTimestamp = +new Date();
                            app.loadAndShowCategoriesPage();
                        }
                    });
                } else {
                    app.loadAndShowCategoriesPage();
                }
            } else {
                app.loadCategories(app.renderCategoriesPage.bind(app));
            }
        },
        loadAndShowCategoriesPage: function(){
            app.user.loadScore(function(){
                app.loadCategories(app.renderCategoriesPage.bind(app));
            });
        },
        showScorePage: function(){
            if(app.onLine) {
                app.user.loadScore(function(){
                    app.loadCategories(app.renderScorePage.bind(app));
                });
            } else {
                app.loadCategories(app.renderScorePage.bind(app));
            }
        },
        showRankingPage: function(){
            if(app.onLine) {
                app.user.loadRanking("points", "global", "last1000", function(){
                    app.renderRankingPage();
                });
            }
        },
        getUserEmail: function(){
            return app.user.user.email;
        },
        getUserID: function(){
            return +app.user.user.userID;
        },
        isAnonymousUser: function(){
            if(localStorage.anonymousUser) {
                var userData = JSON.parse(localStorage.anonymousUser);
                if(this.getUserEmail() === userData.email) return true;
            }

            if(this.getUserID()===-1) return true;

            return false;
        },
        getUnlockedCategories: function(){
            var offlineUnlockedCategories = (localStorage.offlineUnlockedCategories?JSON.parse(localStorage.offlineUnlockedCategories):[]),
                unlcockedCategories = this.categories.filter(function(category){
                    return app.categoriesUnlocked.indexOf(+category.categoryID) !== -1
                            || offlineUnlockedCategories.indexOf(+category.categoryID) !== -1;
                }),

            nextUnlockedIndex = 1;

            unlcockedCategories.forEach(function(category){
                if(category.unlockedIndexNumber && category.unlockedIndexNumber>=nextUnlockedIndex) {
                    nextUnlockedIndex = category.unlockedIndexNumber+1;
                }
            });

            unlcockedCategories.forEach(function(unlockedCategory){
                if(!unlockedCategory.unlockedIndexNumber) {
                    unlockedCategory.unlockedIndexNumber = nextUnlockedIndex;

                    db.categoryStore.get(
                        unlockedCategory.categoryID,
                        function(category){ //success
                            category.unlockedIndexNumber = unlockedCategory.unlockedIndexNumber;
                            db.categoryStore.put(category);
                        }
                    )

                    nextUnlockedIndex++;
                }
            });

            return unlcockedCategories.sort(function(a, b){
                if(app.notPassedCategories.indexOf(a.categoryID) === -1
                    && app.notPassedCategories.indexOf(b.categoryID) !== -1) return -1;

                if(app.notPassedCategories.indexOf(a.categoryID) !== -1
                    && app.notPassedCategories.indexOf(b.categoryID) === -1) return 1;

                if(a.unlockedIndexNumber<b.unlockedIndexNumber) return -1;
                if(a.unlockedIndexNumber>b.unlockedIndexNumber) return 1;
                return 0;
            });
        },
        getLockedCategories: function(){
            var offlineUnlockedCategories = (localStorage.offlineUnlockedCategories?JSON.parse(localStorage.offlineUnlockedCategories):[]);
                lockedCategories =  this.categories.filter(function(category){
                    return app.categoriesUnlocked.indexOf(+category.categoryID) === -1
                            && offlineUnlockedCategories.indexOf(+category.categoryID === -1);
                }).sort(function(a, b){
                    if(a.isSpecial && !b.isSpecial) return -1;
                    if(!a.isSpecial && b.isSpecial) return 1;

                    if(a.indexNumber<b.indexNumber) return -1;
                    if(a.indexNumber>b.indexNumber) return 1;
                    return 0;
                });

            lockedCategories.forEach(function(lockedCategory){
                if(lockedCategory.unlockedIndexNumber) {
                    lockedCategory.unlockedIndexNumber = null;

                    db.categoryStore.get(
                        lockedCategory.categoryID,
                        function(category){
                            category.unlockedIndexNumber = null;
                            db.categoryStore.put(category);
                        }
                    )
                }
            })

            return lockedCategories;
        },
        syncProgress: function(progress){
            app.loader && app.loader.setProgress(progress);
        },
        openCategoryOffline: function(game){
            var unlockedCategories = this.getUnlockedCategories(),
                lastCategory = unlockedCategories[unlockedCategories.length-1],
                offlineUnlockedCategories = (localStorage.offlineUnlockedCategories?JSON.parse(localStorage.offlineUnlockedCategories):[]);

            if(lastCategory.categoryID === game.category.categoryID && game.score.points >= lastCategory.pointsToAvailable) {
                var lockedCategories = this.getLockedCategories();

                if(lockedCategories.length > 0 ){
                    app.categoriesUnlocked.push(lockedCategories[0].categoryID);
                    offlineUnlockedCategories.push(lockedCategories[0].categoryID);
                    localStorage.offlineUnlockedCategories = JSON.stringify(offlineUnlockedCategories);
                    app.markCategoryAsUnplayed(lockedCategories[0].categoryID);
                }
            }
        },
        markCategoryAsUnplayed: function(categoryID) {
            this.notPassedCategories.push(categoryID);
            this.notPassedCategories.unique();
            localStorage.notPassedCategories = JSON.stringify(this.notPassedCategories);
        },
        markCategoryAsPLayed: function(categoryID){
            this.notPassedCategories.removeItem(categoryID);
            localStorage.notPassedCategories = JSON.stringify(this.notPassedCategories);
        },
        categorySynchronizationRequired: function(category){
            return app.synchronization.categories.alreadySynchronizedCategories.filter(function(syncCategory){
                return category.categoryID == syncCategory.categoryID;
            }).length == 0;
        },
        categorySynchronizationAvailable: function(category){
            return app.synchronization.categories.categoriesForSynchonization.filter(function(syncCategory){
                return category.categoryID == syncCategory.categoryID;
            }).length > 0;
        },
        clearBody: function(){
            while(this.appBody.firstChild){
                this.appBody.removeChild(this.appBody.firstChild);
            }
        },
        getScreenHeight: function(){
            if(app.isIOS()){
                return (window.orientation && window.orientation%180) ? screen.width :screen.height
            } else if(app.isAndroid()) {
                return screen.height
            } else {
                return $(window).height();
            }
        },
        getScreenWidth: function(){
            if(app.isIOS()){
                return (window.orientation && window.orientation%180) ? screen.height :screen.width
            } else if(app.isAndroid()) {
                return screen.width
            } else {
                return $(window).width();
            }
        },
        hideCordovaSplashScreen: function(){
            if(window.cordova){
                setTimeout(function(){
                    cordova.exec(null, null, "SplashScreen", "hide", []);
                }, 300);
            }
        }
    }

    augment(App, AppViews);
    augment(App, ScoreViews);
    augment(App, Review);
    augment(App, Tests);

    window.App = App;
})(window);