(function(){
    function Categories(currentVersion){
        this.currentVersion = currentVersion;

        this.categoriesForSynchonization = JSON.parse(localStorage.categoriesForSynchonization||"[]");
        this.alreadySynchronizedCategories = JSON.parse(localStorage.alreadySynchronizedCategories||"[]");
    }

    Categories.prototype.getCategoriesForSynchronization = function(){
        var i, newCategoriesCount = 0, categories = [];

        this.categoriesForSynchonization = this.categoriesForSynchonization.sort(function(a, b){
            if(a.isSpecial && !b.isSpecial) return -1;
            if(!a.isSpecial && b.isSpecial) return 1;

            if(a.indexNumber<b.indexNumber) return -1;
            if(a.indexNumber>b.indexNumber) return 1;
            return 0;
        });

        for(i in this.categoriesForSynchonization) if(this.categoriesForSynchonization.hasOwnProperty(i)){
            var category = this.categoriesForSynchonization[i];

            if(category.syncVersion === this.nextVersion || !category.enabled) {
                continue;
            }

            if(app.categoriesUnlocked.indexOf(category.categoryID) !== -1 && category.syncVersion !== 0) { 
                // All opened categories should synchronize
                categories.push(category);
            } else {
                if(newCategoriesCount<app.startSyncCategoriesCount()) {
                    newCategoriesCount++;
                    categories.push(category);
                }
            }
        }

        return categories;
    }

    Categories.prototype.cloneFields = function(fields, src, dst){
        fields.forEach(function(fieldName){
            src[fieldName] && (dst[fieldName] = src[fieldName]);
        });
    };

    Categories.prototype.addUpdateCategory = function(categoryObject, onSuccess){
        var that = this,
            categoryDataField = "modifiedInfo",
            categoryInfo = categoryObject[categoryDataField];

        if(categoryInfo) {
            categoryInfo.isSound = categoryObject.isSound;
            categoryInfo.backgroundURL = categoryObject.backgroundURL;
            categoryInfo.backgroundExt = categoryObject.backgroundExt;
        } else {
            categoryInfo = {};
        }

        new Category(+categoryObject.id, categoryInfo, function(category){
            if(categoryObject.questionsWasModified) {
                that.markForSynchonization(category);
            }

            onSuccess && onSuccess();
        });
    };

    Categories.prototype.markForSynchonization = function(category){
        var alreadyMarked = this.categoriesForSynchonization.filter(function(alreadyMarked){
            return category.categoryID == alreadyMarked.categoryID;
        });

        if(alreadyMarked.length > 0) {
            this.categoriesForSynchonization.removeItem(alreadyMarked[0]);
        }

        this.categoriesForSynchonization.push({
            categoryID: category.categoryID,
            syncVersion: category.syncVersion,
            isSpecial: category.isSpecial,
            indexNumber: category.indexNumber,
            enabled: category.enabled
        });

        localStorage.categoriesForSynchonization = JSON.stringify(this.categoriesForSynchonization);
    }

    Categories.prototype.markCategoryAsSynchronized = function(category, version){
        var alreadyMarked = this.categoriesForSynchonization.filter(function(alreadyMarked){
            return category.categoryID == alreadyMarked.categoryID;
        });

        if(alreadyMarked.length > 0) {
            this.categoriesForSynchonization.removeItem(alreadyMarked[0]);
        }

        var alreadySynchronized = this.alreadySynchronizedCategories.filter(function(alreadySynchronized){
            return category.categoryID == alreadySynchronized.categoryID;
        });

        if(alreadyMarked.length > 0) {
            this.alreadySynchronizedCategories.removeItem(alreadySynchronized[0]);
        }

        this.alreadySynchronizedCategories.push({
            categoryID: category.categoryID,
            categoryVersion: version
        });

        localStorage.categoriesForSynchonization = JSON.stringify(this.categoriesForSynchonization);
        localStorage.alreadySynchronizedCategories = JSON.stringify(this.alreadySynchronizedCategories);

        category.update(category.categoryID, {
            syncVersion: category.nextVersion
        });
    }

    Categories.prototype.loadCategories = function(callback, skipVersion){
        var that = this
            version = skipVersion?0:this.currentVersion;

        quizAPI.callURL("getCategories", {
            version: version
        }, function(result){
            var data = result.data,
                version = data.version,
                categories = data.categories,
                files = data.files;

            that.nextVersion = version;

            async.parallel(
                categories.map(function(category, index){
                    return function(onCreated){
                        that.addUpdateCategory(category, onCreated);
                    }
                }),
                function(error, results){
                    db.categoryStore.save();
                    localStorage.currentVersion = version;
                    app.synchronization.currentVersion = +version;
                    callback();
                }
            );
        });
    };

    Categories.prototype.loadQuestionsForCategory = function(cid, callback){
        var that = this;
        quizAPI.callURL("getCategoryQuestions", {
                categoryId: cid,
                version: this.currentVersion
            }, function(result){
                var data = result.data,
                    version = data.version,
                    questions = data.questions;

                async.parallel(
                    questions.map(function(question){
                        return function(onCreated){
                            that.addUpdateQuestion(question, onCreated);
                        }
                    }),
                    function(error, results){
                        callback();
                    }
                );
            }
        );
    };

    window.Categories = Categories;
})();