(function(){
    var categoryFields = [
        "name",
        "enabled",
        "isSpecial",
        "pointsToAvailable",
        "useFlagsToAvailable",
        "indexNumber",
        "isSound",
        "syncVersion",
        "backgroundExt",
        "backgroundURL"
    ],
    questionTypes = ["Text","Photo","Sound","Color"],
    answerTypes = ["Text","Photo","Sound","Color"],
    categoryDataField = "modifiedInfo",
    questionDataField = "modifiedInfo";

    function Category(id, modifiedInfo, callback){
        var that = this;
        this.id = id;

        this.progress = 0;

        this.update(id, modifiedInfo, function(){
            that.loadInfo(id, function(){
                callback(that);
            })
        });
    }

    Category.prototype.loadInfo = function(id, callback){
        var that = this,
            fieldName;

        db.categoryStore.get(id, function(category){
            for(fieldName in category) {
                that[fieldName] = category[fieldName];
            }

            callback && callback(that);
        });
    };

    Category.prototype.update = function(categoryID, categoryObject, onSuccess){
        var that = this,
            onSuccess = onSuccess || function(){};

        db.categoryStore.get(categoryID,
            function(category){ //success("Updated")
                that.updateCategory(category, categoryObject, onSuccess);
            },
            function(){ //fail
                that.createCategory(categoryID, categoryObject, onSuccess)
            }
        );
    };

    Category.prototype.createCategory = function(categoryID, categoryObject, onSuccess){
        db.categoryStore.put({
            categoryID: categoryID,
            name: categoryObject.name,
            enabled: categoryObject.enabled,
            pointsToAvailable: +categoryObject.pointsToAvailable,
            useFlagsToAvailable: +categoryObject.useFlagsToAvailable,
            indexNumber: +categoryObject.indexNumber,
            isSpecial: categoryObject.isSpecial,
            isSound: categoryObject.isSound,
            backgroundURL: categoryObject.backgroundURL,
            backgroundExt: categoryObject.backgroundExt,
            syncVersion: 0
        }, onSuccess, onSuccess)
    };

    Category.prototype.updateCategory = function(category, categoryObject, onSuccess){
        this.cloneFields(categoryFields, categoryObject, category);
        db.categoryStore.put(category, onSuccess, onSuccess);
    }

    Category.prototype.cloneFields = function(fields, src, dst){
        fields.forEach(function(fieldName){
            (src[fieldName] !== undefined) && (dst[fieldName] = src[fieldName]);
        });
    };

    Category.prototype.syncCategory = function(onSuccess, onFailure, skipSyncVersion){
        var that = this;

        this.isTerminated = false;

        that.syncQuestions(function(){
            that.syncMedia(onSuccess, onFailure, skipSyncVersion)
        }, onFailure, skipSyncVersion);
    };

    Category.prototype.stopSynchronization = function(){
        this.filesLoader && this.filesLoader.stop();
        this.isTerminated = true;
    };

    Category.prototype.syncQuestions = function(onSuccess, onFailure, skipSyncVersion){
        var that = this,
            questionSynced = false,
            callbackInterval;

        if(skipSyncVersion) {
            this.syncVersion = 0;
        }

        quizAPI.callURL("getCategoryQuestions", {
                categoryId: this.categoryID,
                version: this.syncVersion
            }, function(result){
                var data = result.data,
                    version = data.version,
                    questions = data.questions;

                that.nextVersion = version;

                async.parallel(
                    questions.map(function(question){
                        return function(onCreated){
                            that.syncQuestion(question, onCreated);
                        }
                    }),
                    function(error, results){
                        questionSynced = true;
                    }
                );
            }, function(){
                if(skipSyncVersion) {
                    that.stopSynchronization();
                    onFailure && onFailure();
                } else {
                    that.syncQuestions(onSuccess, onFailure, true);
                }
            }
        );

        callbackInterval = setInterval(function(){
            if(questionSynced){
                clearInterval(callbackInterval);
                onSuccess();
            }
        }, 500);
    };

    Category.prototype.syncQuestion = function(questionObject, onSuccess){
        var that = this;

        if(this.syncVersion==0){
            this.createQuestion(questionObject, onSuccess);
        } else {
            db.questionStore.get(
                questionObject.id,
                function(question){ // success
                    that.updateQuestion(question, questionObject, onSuccess)
                },
                function(){ //fail
                    that.createQuestion(questionObject, onSuccess);
                }
            )
        }
    };

    Category.prototype.createQuestion = function(questionObject, onSuccess){
        db.questionStore.put({
            questionID: +questionObject.id,
            questionType: questionTypes[questionObject[categoryDataField].type],
            question: questionObject[categoryDataField].content,
            answerType: answerTypes[questionObject[categoryDataField].answer_type],
            answer: questionObject[categoryDataField].answer_content,
            level: +questionObject[categoryDataField].level,
            enabled: questionObject[categoryDataField].enabled,
            ready: questionObject[categoryDataField].ready,
            categoryID: questionObject[categoryDataField].category.id,
            tag: (questionObject[categoryDataField].tag)?questionObject[categoryDataField].tag.id:null
        }, onSuccess, onSuccess);
    };

    Category.prototype.updateQuestion = function(question, questionObject, onSuccess){
        var questionData = questionObject[questionDataField];

        (questionData["type"] !== undefined) && (question.questionType = questionTypes[questionData["type"]]);
        questionData.content && (question.question = questionData.content);

        (questionData.answer_type !== undefined) && (question.answerType = answerTypes[questionData.answer_type]);

        (questionData.level !== undefined) && (question.level = questionData.level);

        if(questionData.answer_content) {
            answerContent = questionData.answer_content;
            question.answer = answerContent;
        }

        (questionData.enabled !== undefined) && (question.enabled = questionData.enabled);
        (questionData.ready !== undefined) && (question.ready = questionData.ready);

        if(questionData.category) {
            question.categoryID = +questionData.category.id;
        }

        if(questionData.tag) {
            question.tag = +questionData.tag.id;
        }

        db.questionStore.put(question, onSuccess, onSuccess);
    };

    Category.prototype.syncMedia = function(onSuccess, onFailure, skipSyncVersion){
        var url = quizAPI.getURL("syncCategoryMedia", {
                categoryId: this.categoryID,
                version: this.syncVersion,
                width: 220,
                height: 220,
                audioType: app.player.getAudioType()
            }),
            saveQuestion,
            filesToProcess = [],
            fileLoading = true,
            fileLoadingCount = 1,
            filesLoader,
            version = version,
            that = this,
            questions = [];

        this.filesLoader = filesLoader = new FilesLoader(url, "post", {
                'asl': skipSyncVersion?[]:app.synchronization.categories.alreadySynchronizedCategories
            },
            {
                fileProcessor: function(question, fileBody){
                    filesToProcess.push({
                        crc: question.crc,
                        content: fileBody,
                        fileType: question.fileType
                    });
                },
                questionsProcessor: function(questions){
                    var updateProgress = function(){
                        operationCount = questions.length + filesToProcess.length;

                        that.progress = (doneCount++)/operationCount;
                        app.synchronization.syncProgressUpdate();
                    };

                    fileLoading = false;
                    doneCount = 0;
                    questions = questions;
                    operationCount = questions.length + filesToProcess.length;

                    async.series(questions.map(function(question){
                        var question = question,
                            crc = question.crc;

                        return function(next){
                            if(question.sourceType == "Category") {
                                next();
                            } else {
                                var key = question.prefix + "_" + ((question.sourceType=="Photo")?"image":question.fileType) + "_crc",
                                    info = {};

                                info[key] = crc;

                                db.questionStore.update(question.qid, info, function(){
                                    updateProgress();
                                    next();
                                });
                            }
                        }
                    }), function(){
                        if(that.isTerminated){
                            onFailure && onFailure();
                            onFailure = null;
                        } else {
                            async.series(filesToProcess.map(function(fileData){
                                var fileData = fileData;

                                return function(next){
                                    if(!that.isTerminated) {
                                        app.storage.store(fileData, function(){
                                            updateProgress();
                                            next();
                                        });
                                    }
                                }
                            }), function(){
                                if(that.isTerminated){
                                    onFailure && onFailure();
                                    onFailure = null;
                                } else {
                                    that.progress = 1;
                                    app.synchronization.syncProgressUpdate();
                                    onSuccess();
                                }
                            });
                        }
                    });
                },
                emptyProcessor: function(){
                    if(that.isTerminated){
                        onFailure && onFailure();
                        onFailure = null;
                    } else {
                        that.progress = 1;
                        onSuccess();
                    }
                }
            });
    };

    window.Category = Category;
})();