(function(global){
    var modes = {
        "READ_QUESTIONS": 1,
        "READ_DESCRIPTION": 2,
        "READ_FILE_LENGTH": 3,
        "READ_FILE_BODY": 4
    }

    function FilesLoader(url, method, postData, processors){
        processors = processors || {};

        this.fileProcessor = processors.fileProcessor;
        this.questionsProcessor = processors.questionsProcessor;
        this.emptyProcessor = processors.emptyProcessor;

        this.method = method || 'get';
        this.postData = postData || {};

        this.defineActions();
        this.startProgress(url);
    }

    FilesLoader.prototype = {
        createRequest: function(){
            this.request = new XMLHttpRequest();
        },
        startProgress: function(dataUrl) {
            var that = this;
            this.createRequest();
            this.mode = modes.READ_QUESTIONS;
            this.data = "";
            this.prevDataLength = 0;
            this.startPosition = 0;

            var http = this.request;


            http.open(this.method, dataUrl);
            this.request.setRequestHeader("x-quiz-token", quizAPI.token);
            this.request.setRequestHeader("Content-Type", "application/json; charset=utf-8");
            this.request.overrideMimeType && this.request.overrideMimeType("text/plain; charset=x-user-defined");
            // this.request.responseType = "arraybuffer";

            this.request.onload = function(){
                that.data = this.responseText
                if(this.responseText) {
                    that.parse();
                } else {
                    that.emptyProcessor && that.emptyProcessor();
                }

            }

            http.send(JSON.stringify(this.postData));

        },
        stop: function(){
            this.request.abort();
        },
        parse: function(){
            this.runAction(this.mode);
        },
        runAction: function(mode){
            this.mode = mode;
            this.actions[mode]();
        },
        defineActions: function(){
            var that = this;
            this.actions = {};

            this.actions[modes.READ_QUESTIONS] = function(){
                var data = that.data;
                i = data.indexOf('FILES');

                if(i!==-1) {
                    var lines = data.substring(that.startPosition, i-1).split("\n"),
                        qArray,
                        questions = lines.map(function(line){
                            qArray = line.split("|");
                            return {
                                qid: +qArray[0],
                                sourceType: qArray[1],
                                prefix: qArray[2],
                                fileType: qArray[3],
                                crc: qArray[4]
                            }
                        });

                    that.startPosition = i + "FILES".length + 1;

                    that.questions = questions;
                    that.runAction(modes.READ_DESCRIPTION);
                }
            }

            this.actions[modes.READ_DESCRIPTION] = function(){
                var data = that.data,
                    i = data.indexOf('\n', that.startPosition);

                if(i!==-1) {
                    that.description = data.substring(that.startPosition, i).split("|");
                    that.startPosition = i+1;

                    that.runAction(modes.READ_FILE_LENGTH);
                } else {
                    that.processQuestions(that.questions);
                }
            };

            this.actions[modes.READ_FILE_LENGTH] = function(){
                var data = that.data,
                    i = data.indexOf('\n', that.startPosition);

                if(i!==-1) {
                    that.fileLength = +data.substring(that.startPosition, i);
                    that.startPosition = i+1;

                    that.runAction(modes.READ_FILE_BODY);
                }
            };

            this.actions[modes.READ_FILE_BODY] = function(){
                var data = that.data;

                if(that.startPosition >= data.length){
                    this.finishProgress();
                    return;
                }

                if((data.length - that.startPosition) > that.fileLength) {
                    that.fileBody = data.substring(that.startPosition, that.startPosition+that.fileLength);
                    that.startPosition += (that.fileLength + 1);

                    that.processFile(that.description, that.fileBody);
                    that.runAction(modes.READ_DESCRIPTION);
                }
            };
        },
        processQuestions: function(questions){
            this.questionsProcessor && this.questionsProcessor(questions);
        },
        processFile: function(info, fileBody){
            var info = {
                qid: +info[0],
                sourceType: info[1],
                prefix: info[2],
                fileType: info[3],
                crc: info[4]
            };

            this.fileProcessor && this.fileProcessor(info, fileBody);
        }
    }

    global.FilesLoader = FilesLoader;
})(window);