(function(global){

    function Storage(keyField, storageName){
        this.key = keyField || "id";
        this.storageName = storageName;
        this.storage = {};
    }

    Storage.prototype = {
        toItem: function(item){
            return item;
        },
        get: function(id, onSuccess, onFail){
            var that = this, item;

            if(this.key!=="crc") id = +id;
            item = this.storage[id];

            if(item) {
                onSuccess && onSuccess(item)
            } else {
                onFail && onFail();
            }
        },
        put: function(info, onSuccess, onFail){
            var id = info[this.key],
                item, i;

            item = this.storage[id];

            if(item) {
                for(i in info) if(info.hasOwnProperty(i)) {
                    item[i] = info[i];
                }
            } else {
                item = info;
            }

            this.storage[id] = item;
            onSuccess && onSuccess(item);
            if(!db.syncMode) {
                this.save();
            }
        },
        update: function(id, info, onSuccess){
            var that = this, item, i;

            item = this.storage[id];

            if(item) {
                for(i in info) if(info.hasOwnProperty(i)) {
                    item[i] = info[i];
                }
                this.storage[id] = item;
                onSuccess && onSuccess(item);
            }
            if(!db.syncMode) {
                this.save();
            }
        },
        filter: function(key, value, callback){
            var items = [], that = this, i;

            for (i in this.storage) {
                if (this.storage.hasOwnProperty(i)) {
                    if(this.storage[i][key] == value) {
                        items.push(this.storage[i]);
                    }
                }
            }

            callback(items);
        },
        save: function(callback){
            var that = this;
            localforage.setItem(this.storageName, JSON.stringify(this.storage), function(item, e){
                if(e){
                    console.log(e);
                    if(e.name === 'QuotaExceededError' || e.name === 'NS_ERROR_DOM_QUOTA_REACHED' || e.code === e.QUOTA_ERR){
                        console.log(e);
                        setTimeout(function(){
                            that.save();
                        }, 1000);
                    }
                } else {
                    callback && callback();
                }
            });
        },
        load: function(callback){
            var that = this;
            localforage.getItem(this.storageName, function(data){
                if(data){
                    that.storage = JSON.parse(data);
                }
                callback && callback(data);
            });
        }
    }

    function MemoryStorage(){
        if(navigator.userAgent.match(/(iPod|iPhone|iPad)/)) {
            localforage.setDriver('webSQLStorage');
        }

        localforage.config({
            name        : 'yesnogame-forage-storage',
            version     : 1.0,
            size        : 4980736, // Size of database, in bytes. WebSQL-only for now.
            storeName   : 'foragestorage'
        });

        this.syncMode = false;
        this.userStore = new Storage("userID", "users");
        this.categoryStore = new Storage("categoryID", "categories");
        this.questionStore = new Storage("questionID", "questions");
    }

    MemoryStorage.prototype = {
        saveAll: function(){
            this.userStore.save();
            this.categoryStore.save();
            this.questionStore.save();
        },
        loadAll: function(callback){
            this.userStore.load(function(){
                callback();
                db.categoryStore.load(function(){
                    db.questionStore.load();
                });
            });
        },
        syncStart: function(){
            this.syncMode = true;
        },
        syncFinish: function(){
            this.syncMode = false;
            this.saveAll();
        },
        clear: function(){
            localforage.clear();
        }
    }

    window.MemoryStorage = MemoryStorage;

})(window);