(function(global){
    function joinSameItems(items){
        var updatedItems = [], currentItem, sameItems, updatedItem;

        while(items.length > 0) {
            currentItem = items[0];
            sameItems = items.filter(function(q){
                return q.id === currentItem.id && q !== currentItem;
            });

            sameItems.forEach(function(sameItem){
                for(var property in sameItem.data) {
                    currentItem.data[property] = sameItem.data[property];
                }
                items.removeItem(sameItem);
            });

            updatedItems.push(currentItem);
            items.removeItem(currentItem);
        }
        return updatedItems
    };

    var operationCount = 0,
        doneCount = 0,
        filesCount = 0;

    function Synchronization(){
        this.currentVersion = +localStorage.currentVersion || 0;

        if(("currentScriptVersion" in localStorage) && (+localStorage.currentScriptVersion !== this.scriptVersion)) {
            this.reset();
            this.reloading = true;
            setTimeout(function(){
                window.location.reload();
            }, 1000);
        }

        localStorage.currentScriptVersion = this.scriptVersion;

        this.categories = new Categories(this.currentVersion);
    }

    Synchronization.prototype = {
        scriptVersion: 9,
        syncInitial: function(onReady){
            var that = this,
                categoriesLoaded = false;

            this.onReady = onReady || function(){ app.startApplication(); }
            app.renderSyncPage(true);
            this.syncProgressInit();

            this.syncing = true;
            db.syncStart();

            this.categories.loadCategories(function(){
                that.categoriesLoaded = true;

                that.syncCategories(function(){
                    that.syncReady();
                });
            });
        },
        syncReady: function(){
            this.syncing = false;

            db.syncFinish();
            app.storage.save();

            this.onReady();
        },
        syncCategories: function(callback){
            var categoriesForSynchronization = this.categories.getCategoriesForSynchronization(),
                that = this;

            if(!this.syncing) return;

            this.processingCategories = [];

            async.parallel(
                categoriesForSynchronization.map(function(syncCategoryInfo){
                    return function(onSuccess){
                        var category = new Category(syncCategoryInfo.categoryID, {}, function(category){
                            category.syncCategory(function(){
                                that.categories.markCategoryAsSynchronized(category, category.nextVersion)

                                onSuccess();
                            });
                        });

                        that.processingCategories.push(category);
                    }
                }),
                function(error, results){
                    clearInterval(that.progressInterval);
                    that.syncing = false;
                    callback();
                }
            );
        },
        stopProcessingCategories: function(){
            this.processingCategories && this.processingCategories.forEach(function(category){
                category.stopSynchronization();
            });
        },
        syncProgressInit: function(){
            var startTime = +(new Date()) + 15000,
                endTime = +(new Date()) + 40000,
                that = this;

            setTimeout(function(){
                that.progressInterval = setInterval(function(){
                    var currentTime = +(new Date());

                    if((currentTime - startTime) > 15000 ) {
                        app.syncProgress(1);
                        clearInterval(that.progressInterval);
                        if(that.syncing) {
                            that.stopProcessingCategories();
                            that.syncReady();
                        }
                    } else {
                        if(currentTime > startTime) {
                            app.syncProgress((currentTime - startTime) / 15000);
                        }
                    }
                }, 100);
            }, 10000);

            app.syncProgress(0);
        },
        syncProgressUpdate: function(){
            if(!this.processingCategories) return;

            var progress = 0,
                count = this.processingCategories.length;

            this.processingCategories.forEach(function(category){
                progress += (Math.min(category.progress, 1.0)/count);
            });

            if(progress>this.prevProgress) {
                app.syncProgress(progress);
                this.prevProgress = progress;
            }
        },
        syncCategory: function(id, onSuccess, skipSyncVersion){
            var that = this;

            var category = new Category(id, {}, function(category){
                category.syncCategory(function(){
                    that.categories.markCategoryAsSynchronized(category, category.nextVersion)
                    onSuccess();
                }, function(){ console.log(arguments) }, skipSyncVersion);
            });
            return category;
        },
        checkSyncAvailability: function(callback){
            quizAPI.callURL("checkSync", {
                version: this.currentVersion
            }, function(result){
                callback(result.available);
            });
        },
        saveGame: function(game, callback, offlineCallback){
            var callback = callback || app.showScorePage.bind(app),
                gameInfo = {
                    userId: app.getUserID(),
                    categoryId: game.category.categoryID,
                    points: game.score.points,
                    flags: game.score.flags,
                    questions: game.score.questionsCount,
                    questionsCorrect: game.score.correctQuestionsCount,
                    maxCorrectAnswersSeries: game.score.maxCorrectAnswersSeries,
                    timeSpend: game.score.timeSpend,
                    created: moment().format("DD/MM/YYYY HH:mm:ss")
                },
                that = this;

            app.checkOnline(false,
                function(){
                    that.sendGameToServer(gameInfo, callback);
                },
                function(){
                    that.saveGameLocally(gameInfo, offlineCallback);
                }
            )
        },
        sendGameToServer: function(gameInfo, callback){
            if(gameInfo.userId === -1) {
                callback && callback();
            } else {
                quizAPI.callURL("saveGame", gameInfo, callback);
            }
        },
        saveGameLocally: function(gameInfo, callback){
            var games = [];
            if(localStorage.games) {
                games = JSON.parse(localStorage.games);
            }

            games.push(gameInfo);

            localStorage.games = JSON.stringify(games);
            callback();
        },
        sendLocalGamesToServer: function(){
            if(localStorage.games) {
                var games = JSON.parse(localStorage.games);

                games.forEach(function(gameInfo){
                    this.sendGameToServer(gameInfo);
                }, this);

                delete localStorage.games;
            }
        },
        reset: function(){
            this.currentVersion = 0;

            delete localStorage.currentVersion;
            delete localStorage.currentUserID;
            delete localStorage.anonymousUser;
            delete localStorage.games;
            delete localStorage.categoriesWithSound;
            delete localStorage.notPassedCategories;
            delete localStorage.categoriesUnlocked;
            delete localStorage.currentScriptVersion;
            delete localStorage.categoriesForSynchonization;
            delete localStorage.alreadySynchronizedCategories;
            delete localStorage.secondVisit;
            delete localStorage.assetsPortions;
            delete localStorage.offlineUnlockedCategories;
            delete localStorage.reviewQuestions;

            db.clear();
        }
    }

    global.Synchronization = Synchronization;

})(window);