(function(global){

    function Storage(entity, keyField, fields){
        this.Entity = entity;
        this.key = keyField || "id";
        this.fields = fields || [];
    }

    Storage.prototype = {
        toItem: function(item){
            var data = item.data;
            this.fields.forEach(function(fieldName){
                data[fieldName] = item[fieldName];
            });

            return data;
        },
        get: function(id, onSuccess, onFail){
            var that = this;

            if(this.key!=="crc") id = +id;

            this.Entity.findBy(this.key, id, function(result){
                if(!!result) {
                    onSuccess && onSuccess(that.toItem(result));
                } else {
                    onFail && onFail();
                }
            });
        },
        put: function(info, onSuccess, onFail){
            var id = info[this.key],
                that = this, item,
                fields = {};

            this.fields.forEach(function(fieldName){
                fields[fieldName] = info[fieldName]
                delete info[fieldName];
            });

            item = new this.Entity({
                data: info
            });

            this.fields.forEach(function(fieldName){
                item[fieldName] = fields[fieldName];
            });
            item[this.key] = info[this.key];

            persistence.add(item);
            onSuccess && onSuccess(that.toItem(item));
        },
        update: function(id, info, onSuccess){
            var that = this;

            if(this.key!=="crc") id = +id;

            this.Entity.findBy(this.key, id, function(result){
                if(!!result) {
                    var key;
                    for(key in info) if(info.hasOwnProperty(key)){
                        result.data[key] = info[key];
                    }

                    result.markDirty("data");

                    persistence.flush(function(){
                        onSuccess && onSuccess(that.toItem(result));
                    })

                } else {
                    onFail && onFail();
                }
            });
        },
        filter: function(key, value, callback){
            var items = [], that = this;

            this.Entity.all().list(function(items){
                var result = items.filter(function(item){
                    return (item.data[key] == value);
                }).map(function(item){
                    return that.toItem(item)
                });

                callback(result);
            });
        }
    }

    function WebSQLStorage(){
        var sqldriver = window.cordova?persistence.store.cordovasql:persistence.store.websql;
        this.websql = true;
        persistence.debug = true;

        sqldriver.config(
            persistence,
            'yesnogame',
            '1.0',                // DB version
            'YesNoGame',          // DB display name
            5 * 1024 * 1024,        // DB size
            0                       // SQLitePlugin Background processing disabled
        );

        this.defineScheme();

        this.userStore = new Storage(this.User, "userID");
        this.categoryStore = new Storage(this.Category, "categoryID");
        this.questionStore = new Storage(this.Question, "questionID");
        this.fileStore = new Storage(this.Asset, "crc", ["content"]);

    }

    WebSQLStorage.prototype = {
        defineScheme: function(){
            this.User = persistence.define('User', {
                userID: "INT",
                data: "JSON"
            });

            this.User.index("userID", {unique:true});

            this.Category = persistence.define("Category", {
                categoryID: "INT",
                data: "JSON"
            });
            this.Category.index("categoryID", {unique:true});

            this.Question = persistence.define('Question', {
                questionID: "INT",
                data: "JSON"
            });
            this.Question.index("questionID", {unique:true});

            this.Asset = persistence.define('Asset', {
                crc: "TEXT",
                content: "TEXT",
                data: "JSON"
            });
            this.Asset.index("crc", {unique:true});

            persistence.schemaSync();
        },
        clear: function(){
            persistence.reset()
        }
    }

    window.WebSQLStorage = WebSQLStorage;

})(window);