(function(global){

    var currentQuestionSet = [],
        lastQuestions = [],
        lastAnswers= [];

    function toBase64(fileBody) {
        fileBody  = base32k.decodeBytes(fileBody);
        var bytes = [];
        for (var i = 0, len = fileBody.length; i < len; ++i) {
          var c = fileBody.charCodeAt(i);
          bytes.push(c);
        }

        return btoa(String.fromCharCode.apply(null, new Uint8Array(bytes)));
    }

    function GameQuestions(questions){
        this.questions = questions;
        currentQuestionSet = questions;

        this.questionsByLevel = {
            1: questions.filter(function(question){
                return question.level < 2;
            }),
            2: questions.filter(function(question){
                return question.level < 3;
            }),
            3: questions.filter(function(question){
                return question.level < 4;
            })
        }
    }

    GameQuestions.prototype = {
        getNextQuestion: function(){
            var questions = this.questionsByLevel[app.game.getLevel()],
                retryCount = 0;

            question = questions[Math.floor(Math.random() * questions.length)];

            while(lastQuestions.indexOf(question.questionID) !== -1 && retryCount < 5) {
                question = questions[Math.floor(Math.random() * questions.length)];
            }

            lastQuestions.push(question.questionID);

            if(lastQuestions.length > 5) {
                lastQuestions.shift();
            }

            return new Question(question);
        },
        checkIntegrity: function(){
            if(this.questions.length === 0) return false;

            for(var i=0; i<this.questions.length; i++) {
                var question = this.questions[i];

                if(question.questionType == "Photo") {
                    var ext = question.question["type"],
                        key = question.question_image_crc + "." + ext;

                    if(!question.question_image_crc || !app.storage.hasKey(key)) {
                        return false;
                    }
                }

                if(question.answerType == "Photo") {
                    var ext = question.answer["type"],
                        key = question.answer_image_crc + "." + ext;

                    if(!question.answer_image_crc || !app.storage.hasKey(key)) {
                        return false;
                    }
                }

                if(question.questionType == "Sound") {
                    if("question_mp3_crc" in question) {
                        var key = question.question_mp3_crc + ".mp3";
                        if(!question.question_mp3_crc || !app.storage.hasKey(key)) {
                            return false;
                        }
                    }

                    if("question_ogg_crc" in question) {
                        var key = question.question_ogg_crc + ".ogg";
                        if(!question.question_ogg_crc || !app.storage.hasKey(key)) {
                            return false;
                        }
                    }
                }

                if(question.answerType == "Sound") {
                    if("answer_mp3_crc" in question) {
                        var key = question.answer_mp3_crc + ".mp3";
                        if(!question.answer_mp3_crc || !app.storage.hasKey(key)) {
                            return false;
                        }
                    }

                    if("answer_ogg_crc" in question) {
                        var key = question.answer_ogg_crc + ".ogg";
                        if(!question.answer_ogg_crc || !app.storage.hasKey(key)) {
                            return false;
                        }
                    }
                }
            }

            return true;
        }
    }

    function Question(dbQuestion){
        this.dbQuestion = dbQuestion;

        this.correct = (Math.random() < 0.5);

        if(!this.correct) {
            this.questionsWithSameTag = this.selectQuestionsWithSameTag();

            if(this.questionsWithSameTag.length < 2) {
                this.correct = true;
            }
        }

        this.render();
    }

    Question.prototype = {
        render: function(){
            this.element = dom(".container.question-container", [
                this.correctAnswerElement = dom("span.answer-animation.fa.fa-check"),
                this.wrongAnswerElement = dom("span.answer-animation.fa.fa-times")
            ]);

            this.renderQuestion();
            this.renderAnswer();

            return this.element;
        },
        selectQuestionsWithSameTag: function(){
            var that = this;

            return currentQuestionSet.filter(function(question){
                return question !== that.dbQuestion &&
                    question.answer != that.dbQuestion.answer &&
                    question.questionType === that.dbQuestion.questionType &&
                    question.answerType === that.dbQuestion.answerType &&
                    question.tag === that.dbQuestion.tag
            });
        },
        generateWrongAnswer: function(){
            var retryCount = 0,
                question = this.questionsWithSameTag[Math.floor(Math.random()*this.questionsWithSameTag.length)];

            while(lastQuestions.indexOf(question.questionID) !== -1 && retryCount<5) {
                question = this.questionsWithSameTag[Math.floor(Math.random()*this.questionsWithSameTag.length)];
                retryCount++;
            }

            return question;
        },
        sayYes: function(){
            if(this.answered) return;
            this.answered = true;

            if(this.correct) {
                this.correctAnswer();
            } else {
                app.saveQuestionForReview(this.dbQuestion, this.otherQuestion);
                this.uncorrectAnswer();
            }
        },
        sayNo: function(){
            if(this.answered) return;
            this.answered = true;

            if(this.correct) {
                app.saveQuestionForReview(this.dbQuestion);
                this.uncorrectAnswer();
            } else {
                this.correctAnswer();
            }
        },
        correctAnswer: function(){
            this.element.addClass("correct");
            if(app.isIOS()){
                this.element.addClass("fast-fade-out");
            } else {
                this.element.addClass("fade-out");
            }

            setTimeout(function(){
                app.game.correctAnswer();
            }, app.isIOS()?300:500);
        },
        uncorrectAnswer: function(){
            this.element.addClass("wrong");
            if(app.isIOS()){
                this.element.addClass("fast-fade-out");
            } else {
                this.element.addClass("fade-out");
            }

            setTimeout(function(){
                app.game.uncorrectAnswer();
            }, app.isIOS()?300:500);
        },
        renderQuestion: function(){
            var dataType = this.dbQuestion.questionType,
                value = this.dbQuestion.question;

            this.questionElement = this["render" + dataType](value, this.dbQuestion);

            this.element.appendChild(this.questionElement);
        },
        renderAnswer: function(){
            var otherQuestion, dataType, answer;

            if(!this.correct) {
                otherQuestion = this.generateWrongAnswer();
                dataType = otherQuestion.answerType;
                answer = otherQuestion.answer;

                lastQuestions.push(otherQuestion.questionID);

                if(lastQuestions.length > 5) {
                    lastQuestions.shift();
                }
            } else {
                dataType = this.dbQuestion.answerType,
                answer = this.dbQuestion.answer;
            }

            this.otherQuestion = otherQuestion;

            this.answerElement = this["render" + dataType](answer, otherQuestion||this.dbQuestion, "answer");

            this.element.appendChild(this.answerElement);
        },
        renderColor: function(value){
            return dom(".color.item", {style: "background: " + value + ";"});
        },
        renderText: function(value){
            return dom(".item.number", value);
        },
        renderPhoto: function(photo, question, prefix){
            var that = this,
                prefix = prefix || "question",
                element =  dom(".item.image", [
                    this.image = dom(".img")
                ]);

            app.storage.get(question[prefix + "_image_crc"], photo["type"], function(imageSrc){
                that.image.setAttribute("style", "background: url(" + imageSrc + ") center center no-repeat; background-size: contain;");
            });

            return element;
        },
        renderSound: function(sound, question, prefix){
            var that=this,
                playButton,
                prefix = prefix || "question",
                element;

            element = dom(".item.sound", {}, [
                playButton = dom("i.fa.fa-volume-up")
             ]);

            app.player.playQuestion(question, prefix);

            element.addEventListener("click", function(){
                app.player.playQuestion(question, prefix);
            });

            return element;
        }
    }

    window.GameQuestions = GameQuestions;
})(window);