(function(global){

    function Game(category){
        this.category = category;

        this.timerWidget = new Timer();
        this.bonusWidget = new Bonus();
        this.pointsWidget = new Points();
        this.heartsWidget = new Hearts();

        this.refreshQuestions();
    }

    Game.prototype = {
        refreshQuestions: function(callback){
            var questions = [],
                game = this;

            this.questions = null;

            db.questionStore.filter("categoryID", game.category.categoryID, function(questions){
                questions = questions.filter(function(question){
                    return question.enabled;
                });

                game.questions = new GameQuestions(questions);

                game.play();
                callback && callback();
            });
        },
        render: function(){
            var that = this;
            app.clearBody();

            this.element = dom(".game-page", [
                dom(".game-header-wrapper", [
                    this.header = dom("table.header", [
                        dom("tr", [
                            dom("th.timer", [
                                dom("span", _("time"))
                            ]),
                            dom("th.bonus", [
                                dom("span", _("bonus"))
                            ]),
                            dom("th.points", [
                                dom("span", _("points"))
                            ]),
                            dom("th.hearts", [
                                dom("span", _("lives"))
                            ])
                        ]),
                        dom("tr", [
                            dom("td.timer", [this.timerWidget.element]),
                            dom("td.bonus", [this.bonusWidget.element]),
                            dom("td.points", [this.pointsWidget.element]),
                            dom("td.hearts", [this.heartsWidget.element])
                        ]),
                        dom("tr", [
                            dom("td", { colspan: "4" }, [
                                this.categoryLabel = dom(".active-category", this.category.name)
                            ])
                        ])
                    ]),
                ]),
                this.questionBody = dom(".question-body"),
                this.answers = dom(".answers", [
                    this.progress = dom(".progress"),
                    this.yesButton = dom("span.yes", _("YES")),
                    this.noButton = dom("span.no", _("NO"))
                ])
            ]);

            if(this.category.name.length > 13) {
                this.categoryLabel.addClass("long-name");
            }

            this.yesButton.addEventListener("click", this.sayYes.bind(this));
            this.noButton.addEventListener("click", this.sayNo.bind(this));

            app.appBody.appendChild(this.element);

            this.recalculateQuestionHeight();

            window.addEventListener('resize', this.recalculateQuestionHeight.bind(this));
            window.addEventListener('orientationchange', this.recalculateQuestionHeight.bind(this));
        },
        setBackground: function(){
            if(app.onLine && this.category.isSpecial) {
                var game = this, image,
                    backgroundURL = quizAPI.getURL("specialCategoryBackground", {
                        width: app.getScreenWidth(),
                        height: app.getScreenHeight(),
                        categoryID: this.category.categoryID
                    });

                image = new Image();

                image.onload = function(){
                    if(image.width > 1 && image.height > 1){
                        game.element.style.background = "url("+ backgroundURL + ")";
                        game.element.style.backgroundPosition = "center center";
                        game.element.style.backgroundSize = "cover";

                        setTimeout(function(){
                            game.element.addClass("special");
                        }, 300);
                    }
                }

                image.src = backgroundURL;
            }
        },
        clearBackground: function(){
            this.element.style.background = "";
            this.element.style.backgroundPosition = "";
            this.element.style.backgroundSize = "";
        },
        recalculateQuestionHeight: function(){
            var height = $(window).height() - $(this.header).height() - $(this.answers).height();
            $(this.questionBody).height(height);
        },
        sayYes: function(){
            this.currentQuestion.sayYes();
        },
        sayNo: function(){
            this.currentQuestion.sayNo();
        },
        play: function(){
            this.correct = 0;
            this.maxCorrectAnswers = 0;
            this.uncorrect = 0;

            this.initScore();
            this.render();
            this.renderStartPage();
            this.setBackground();
        },
        initScore: function(){
            this.score = {
                points: 0,
                flags: 0,
                questionsCount: 0,
                correctQuestionsCount: 0,
                maxCorrectAnswersSeries: 0,
                timeSpend: 60
            }
        },
        renderStartPage: function(){
            var button, goalGroup, goalLabel, startButton, game = this;
            this.questionBody.innerHTML = "";

            var goalGroup = dom(".goal-group", [
                goalLabel = dom(".goal-label", _("Goal: ") + this.category.pointsToAvailable + _(" points")),
                startButton = dom(".lets-go", _("Go!")),
                cancelButton = dom(".cancel-button", _("Cancel"))
            ]);

            this.questionBody.appendChild(goalGroup);

            startButton.addEventListener("click", function(){
                if(!game.questions.checkIntegrity()) {
                    app.syncAndStartGame(game.category, true);
                } else {
                    game.startPlaying();
                }
            });

            if(!app.doNotTrackHistory) {
                window.history.pushState({
                    page: "renderStartPage",
                    scope: "game"
                }, "Yes No Game");
            } else {
                app.doNotTrackHistory = false;
            }

            cancelButton.addEventListener("click", app.showCategoriesPage.bind(app));

            if(this.category.name === "start" && !("doNotShowStartDialog" in localStorage)) {
                var checkbox;

                dialog(
                    "START",
                    dom("div", [
                        dom("span.left", _("If you score the same number of points as the category goal, you unlock the next category.")),
                        dom(".not-show-again.left", [
                            dom(".checkbox", [
                                checkbox = dom("input#notShowAgainCheckbox", { type: "checkbox" }),
                                dom("label", {"for": "notShowAgainCheckbox"})
                            ]),
                            dom("span", _("Don't display again"))
                        ])
                    ]),
                    [{
                        value: "OK",
                        callback: function(){
                            if(checkbox.checked) {
                                localStorage.doNotShowStartDialog = "true";
                            }
                        }
                    }]
                );
            }
        },
        startPlaying: function(){
            var game = this;
            app.appBody.addClass("playing");
            game.progress.addClass("animate");

            document.addEventListener("keydown", game.arrowsHandler = function(e){
                e = e || window.event;

                if (e.keyCode == '37') { // left
                    game.sayYes();
                }
                else if (e.keyCode == '39') { // right
                    game.sayNo();
                }
            });

            game.renderQuestion();
            game.timerWidget.tick();
        },
        isCategoryNew: function(){
            return app.notPassedCategories.indexOf(this.category.categoryID) !== -1;
        },
        getGameOverMessage: function(){
            if(this.isCategoryNew()) {
                if(this.score.points >= this.category.pointsToAvailable) {
                    return _("Well done!") + "<br>"
                        + _("You've unlocked") + "<br>"
                        + _("the next category!");

                } else if(this.heartsWidget.value > 0) {
                    return _("Too bad! ") + (this.category.pointsToAvailable - this.score.points) + " points" + "<br/>"
                        + _("missing to unlock next")+ "<br/>"
                        + _("category. Try Again!");
                } else {
                    return _("Sorry! You've") + "<br>" +
                        _(" lost your 3 lives.") + "<br>" +
                        _("Start again!");
                }
            } else {
                if(this.score.points >= this.category.pointsToAvailable) {
                    return _("Super! ") + (this.score.points - this.category.pointsToAvailable) + _(" points") + "<br>" + _(" over the goal!");
                } else {
                    return (this.category.pointsToAvailable - this.score.points) + _(" points") + "<br>" + _(" under the goal!");
                }
            }
        },
        renderGameOverPage: function(){
            var game = this,
                goalLabel, gameOverLabel, gameOverMessage, buttons, replayButton, newGameButton, scoreButton;

            this.questionBody.innerHTML = "";

            goalLabel = dom(".goal-label-game-over", _("Goal: ") + this.category.pointsToAvailable + _(" points"))
            gameOverMessage = dom(".game-over-message", this.getGameOverMessage());

            this.buttons = dom(".go-buttons.disabled", [
                replayButton = dom(".go.button", _("Replay")),
                newGameButton = dom(".go.button",_("New game")),
                scoreButton = dom(".go.button", _("Score"))
            ]);

            setTimeout(function(){
                game.buttons.removeClass("disabled")
            }, 1000);

            this.questionBody.appendChild(goalLabel);
            this.questionBody.appendChild(gameOverMessage);
            this.questionBody.appendChild(this.buttons);

            setTimeout(function(){
                gameOverMessage.addClass("animation");
            }, 10);

            replayButton.addEventListener("click", function(e){
                app.appBody.removeClass("playing");
                if(app.isAnonymousUser()){
                    app.user.renderJoinPage(e, function(){
                        app.startGame(game.category);
                    });
                } else {
                    app.startGame(game.category);
                }

            });

            newGameButton.addEventListener("click", function(e){
                app.appBody.removeClass("playing");
                if(app.isAnonymousUser()){
                    app.user.renderJoinPage(e, function(){
                        app.showCategoriesPage();
                    });
                } else {
                    app.showCategoriesPage();
                }
            });

            scoreButton.addEventListener("click", function(){
                app.appBody.removeClass("playing");
                app.showScorePage();
            });

            if(this.isCategoryNew() && this.score.points >= this.category.pointsToAvailable) {
                app.markCategoryAsPLayed(this.category.categoryID);
                game.showShareButton(this.category.name);
            }
        },
        showShareButton: function(categoryName){
            var shareWrapper, shareButton;

            shareWrapper = dom(".share-wrapper", [
                shareButton = dom(".button.share-button", [
                    dom("span", _("Share with friends")),
                    dom("span.fa.fa-facebook-square"),
                    dom("span.fa.fa-facebook")
                ])
            ]);

            this.questionBody.insertBefore(shareWrapper, this.buttons);
            shareButton.addEventListener("click", this.shareResult.bind(this));
        },
        shareResult: function(){
            if(window.cordova) {
                 if(!app.signedInWithFacebook) {
                    FB.login(['email', 'user_birthday', 'user_friends'], function(){
                        app.signedInWithFacebook = true;
                        app.game.shareResult();
                    });
                } else {
                    FB.showDialog({
                        method: "feed",
                        link: settings.achievementURL + "?category_name=" + this.openedCategoryName.toUpperCase(),
                    }, function(){
                        console.log(arguments)
                    }, function(){
                        console.log(arguments)
                    });
                }
            } else {
                FB.ui({
                    method: 'share',
                    href: settings.achievementURL + "?category_name=" + this.openedCategoryName.toUpperCase()
                }, function(response){
                    console.log(response);
                });
            }
        },
        renderQuestion: function(){
            app.player.pause();

            if(this.gameIsOver) return;
            this.currentQuestion = this.questions.getNextQuestion();
            this.questionBody.innerHTML = "";
            this.questionBody.appendChild(this.currentQuestion.element);
        },
        correctAnswer: function(){
            this.correct ++;
            this.maxCorrectAnswers ++;

            this.score.questionsCount++;
            this.score.correctQuestionsCount++;

            if(this.maxCorrectAnswers > this.score.maxCorrectAnswersSeries) {
                this.score.maxCorrectAnswersSeries = this.maxCorrectAnswers;
            }

            this.updatePoints();
            this.updateBonus();

            this.renderQuestion();
        },
        uncorrectAnswer: function(){
            this.correct = 0;
            this.maxCorrectAnswers = 0;

            this.score.questionsCount++;

            this.bonusWidget.dec();
            this.heartsWidget.dec();

            if(this.heartsWidget.value == 0) {
                this.gameOver();
            } else {
                this.renderQuestion();
            }
        },
        updateBonus: function(){
            if(this.correct>=5) {
                this.bonusWidget.inc();
                this.score.flags += 2;
                this.correct = 0;
            }
        },
        updatePoints: function(){
            if(!this.gameIsOver) {
                this.pointsWidget.inc(this.bonusWidget.getBonus());
                this.score.points = this.pointsWidget.points;
            }
        },
        gameOver: function(){
            var game = this;
            this.gameIsOver = true;

            app.player.pause();
            this.clearBackground();

            this.progress.removeClass("animate");
            this.timerWidget.stop();

            document.removeEventListener("keydown", this.arrowsHandler);
            this.answers.parentNode.removeChild(this.answers);

            game.renderGameOverPage();
            game.openedCategoryName = game.category.name;

            app.synchronization.saveGame(this,
                function(response){
                    if(response && response.data && response.data.categoryUnlocked) {
                        var categoryId = +response.data.categoryUnlocked,
                            openedCategoryName = app.categories.filter(function(c){
                                return c.categoryID==categoryId
                            })[0].name;
                    }
                },
                function(){
                    app.openCategoryOffline(game);
                }
            );

            app.saveReviewBundle();

            if(this.onEnd) {
                this.onEnd();
                this.onEnd = null;
            }
        },
        getLevel: function(){
            return Math.min(this.bonusWidget.value, 3);
        }
    }

    function Timer(time){
        this.time = time || 45;
        this.render();
    }

    Timer.prototype = {
        render: function(){
            this.element = dom("span", {
                innerHTML: this.time.toMSS()
            })
        },
        setTime: function(time){
            this.time = time;
            this.element.innerHTML = this.time.toMSS();


        },
        tick: function(){
            var that = this;
            this.tickTimeout = setTimeout(function(){
                that.dec();
                that.tick();
            }, 1000);
        },
        dec: function(){
            if(this.time>0 && !this.gameOver){
                this.setTime(this.time-1)
            } else {
                (!this.gameOver) && app.game.gameOver();
            }
        },
        stop: function(){
            this.gameOver = true;
        },
        pauseOn: function(){
            var computedStyle = window.getComputedStyle(app.game.progress),
                progressWidth = computedStyle.getPropertyValue('width');

            clearTimeout(this.tickTimeout);

            app.game.progress.style.width = progressWidth;
            app.game.progress.removeClass("animate");
        },
        pauseOff: function(){
            app.game.progress.addClass("animate");
            app.game.progress.style.removeProperty("width");
            var transitionDurationProperty = getStyleProperty('transitionDuration');
            app.game.progress.style[transitionDurationProperty] = (this.time*1000) + "ms";
            this.tick();
        }

    }

    var bonus_values = [10, 11, 13, 16, 20]

    function Bonus(value){
        this.value = value||1;
        this.render();
    }

    Bonus.prototype = {
        render: function(){
            this.element = dom("div", this.stars = [
                dom(".fa.fa-star", {"data-bonus": bonus_values[0]}),
                dom(".fa.fa-star", {"data-bonus": bonus_values[1]}),
                dom(".fa.fa-star", {"data-bonus": bonus_values[2]}),
                dom(".fa.fa-star", {"data-bonus": bonus_values[3]}),
                dom(".fa.fa-star", {"data-bonus": bonus_values[4]})
            ]);

            this.refreshStars();
        },
        refreshStars: function(){
            this.stars.forEach(function(starElement, i){
                starElement.removeClass("last-active");
                if(this.value>i) {
                    starElement.addClass("active");
                    if(this.value == i+1) {
                        starElement.addClass("last-active");
                    }
                } else {
                    starElement.removeClass("active");
                }
            }, this);
        },
        getBonus: function(){
            return bonus_values[this.value-1];
        },
        setValue: function(value){
            this.value = value;
            this.refreshStars();
        },
        inc: function(){
            if(this.value<5){
                this.setValue(this.value+1);
            }
        },
        dec: function(){
            if(this.value>1){
                this.setValue(this.value-1);
            }
        }
    }

    function Hearts(hearts) {
        this.value = hearts || 3;
        this.render();
    }

    Hearts.prototype = {
        render: function(){
            this.element = dom("div", {}, this.hearts = [
                dom(".fa.fa-heart-o"),
                dom(".fa.fa-heart-o"),
                dom(".fa.fa-heart-o")
            ]);

            this.refreshHearts();
        },
        refreshHearts: function(){
            this.hearts.forEach(function(heartElement, i){
                if(this.value>i) {
                    heartElement.addClass("active");
                } else {
                    heartElement.removeClass("active");
                }
            }, this);
        },
        dec: function(){
            if(this.value) {
                this.value = this.value - 1;
                this.refreshHearts();
            }
        }
    }

    function Points(points){
        this.points = points || 0;
        this.render();
    }

    Points.prototype = {
        render: function(){
            this.element = dom("span", {innerHTML: this.points})
        },
        setPoints: function(points){
            this.points = points;
            this.element.innerHTML = this.points;
        },
        inc: function(points){
            this.setPoints(this.points + points);
        }
    }

    window.Game = Game;
})(window);