(function(global){
    var questionTypes = ["Text","Photo","Sound","Color"],
        answerTypes = ["Text","Photo","Sound","Color"];

    function Review(){}

    Review.prototype = {
        showOrHideReviewButton: function(){
            if(app.reviewButton) {
                if(app.isReviewAvailable()) {
                    app.reviewButton.addClass("active");
                } else {
                    app.reviewButton.removeClass("active")
                }
            }
        },
        syncReviewQuestions: function(callback){
            var userID = app.getUserID();

            quizAPI.callURL("loadReviewQuestions", {
                userID: userID
            }, function(result){
                var reviewQuestions = [],
                    syncRequiredQuestions = [];

                result.data.forEach(function(question){
                    if(reviewQuestions.indexOf(question.questionId) === -1) {
                        reviewQuestions.push(question.questionId);
                    }

                    if(question.text && question.text.length>0) {
                        try {
                            var meta = JSON.parse(question.text);

                            if(meta.otherQuestionId && reviewQuestions.indexOf(meta.otherQuestionId)) {
                                reviewQuestions.push(meta.otherQuestionId);
                            }
                        } catch(e) {
                            console.log("Parse error:")
                            console.log(question);
                        }
                    }
                });

                async.series(reviewQuestions.map(function(questionID){
                    var questionID = questionID;

                    return function(next){

                        db.questionStore.get(
                            questionID,
                            function(dbQuestion){ //success
                                if(dbQuestion.questionType !== "Photo"
                                    && dbQuestion.questionType !== "Sound"
                                    && dbQuestion.answerType !== "Photo"
                                    && dbQuestion.answerType !== "Sound") {

                                    next();
                                } else {
                                    if(dbQuestion.questionType === "Photo") {
                                        var ext = dbQuestion.question["type"],
                                            key = dbQuestion.question_image_crc + "." + ext;

                                        if(!app.storage.hasKey(key)) {
                                            syncRequiredQuestions.push(questionID);
                                        }
                                    }

                                    if(dbQuestion.questionType === "Sound") {
                                        if("question_mp3_crc" in dbQuestion) {
                                            var key = dbQuestion.question_mp3_crc + ".mp3";
                                            if(!app.storage.hasKey(key)) {
                                                syncRequiredQuestions.push(questionID);
                                            }
                                        }

                                        if("question_ogg_crc" in dbQuestion) {
                                            var key = dbQuestion.question_ogg_crc + ".ogg";
                                            if(!app.storage.hasKey(key)) {
                                                syncRequiredQuestions.push(questionID);
                                            }
                                        }
                                    }

                                    next();
                                }
                            },
                            function(){
                                syncRequiredQuestions.push(questionID);
                                next();
                            }
                        )
                    }
                }), function(){
                    var url = quizAPI.getURL("syncReviewMedia", {
                        userID: userID,
                        width: 220,
                        height: 220,
                        audioType: app.player.getAudioType()
                    }),
                    saveQuestion,
                    filesToProcess = [],
                    reviewLoader,
                    that = this,
                    questions = [];

                    this.reviewLoader = reviewLoader = new ReviewLoader(url, "post", {
                            questions: syncRequiredQuestions
                        },
                        {
                            fileProcessor: function(question, fileBody){
                                filesToProcess.push({
                                    crc: question.crc,
                                    content: fileBody,
                                    fileType: question.fileType
                                });
                            },
                            questionsProcessor: function(questions){
                                questions.forEach(function(questionObject){
                                    db.questionStore.get(
                                        questionObject.questionID,
                                        function(question){ // success
                                            // Do nothing, as it is bad idea to modify existing question
                                        },
                                        function(){ //fail
                                            db.questionStore.put({
                                                questionID: +questionObject.questionID,
                                                questionType: questionTypes[+questionObject.questionType],
                                                question: questionObject.question,
                                                answerType: answerTypes[+questionObject.answerType],
                                                answer: questionObject.answer,
                                                categoryID: questionObject.categoryID
                                            });
                                        }
                                    )
                                });
                            },
                            mediaQuestionsProcessor: function(questions){
                                var questions = questions;

                                async.series(questions.map(function(question){
                                    var question = question,
                                        crc = question.crc;

                                    return function(next){
                                        if(question.sourceType == "Category") {
                                            next();
                                        } else {
                                            var key = question.prefix + "_" + ((question.sourceType=="Photo")?"image":question.fileType) + "_crc",
                                                info = {};

                                            info[key] = crc;
                                            if(question.prefix=="question") {
                                                info.question = { "type": question.fileType };
                                            } else {
                                                info.answer = { "type": question.fileType };
                                            }

                                            db.questionStore.update(question.qid, info, function(){
                                                next();
                                            });
                                        }
                                    }
                                }), function(){
                                    async.series(filesToProcess.map(function(fileData){
                                        var fileData = fileData;

                                        return function(next){
                                            app.storage.store(fileData, function(){
                                                next();
                                            });
                                        }
                                    }), function(){
                                        app.reviewQuestions = result.data;
                                        localStorage.reviewQuestions = JSON.stringify(app.reviewQuestions);

                                        callback && callback();
                                    });
                                });
                            },
                            emptyProcessor: function(){
                                app.reviewQuestions = result.data;
                                localStorage.reviewQuestions = JSON.stringify(app.reviewQuestions);

                                callback && callback();
                            }
                        });
                });
            });
        },
        loadReviewQuestions: function(callback){
            if("reviewQuestions" in localStorage) {
                this.reviewQuestions = JSON.parse(localStorage.reviewQuestions);
            } else {
                this.reviewQuestions = [];
            }
        },
        clearReviewQuestions: function(){
            delete localStorage.reviewQuestions;
            this.reviewQuestions = [];
        },
        saveQuestionForReview: function(question, otherQuestion){
            var metaData = {
                    categoryName: app.game.category.name
                },
                reviewData;

            if(!this.reviewQuestions) {
                this.reviewQuestions = [];
            }

            if(otherQuestion) {
                metaData.wrongAnswer = otherQuestion.answer;
                metaData.otherQuestionId = otherQuestion.questionID;
            }

            this.reviewQuestions.unshift({
                questionId: question.questionID,
                text: JSON.stringify(metaData)
            });

            if(!this.reviewBundle) {
                this.reviewBundle = [{
                    id: question.questionID,
                    text: JSON.stringify(metaData)
                }];
            } else {
                this.reviewBundle.unshift({
                    id: question.questionID,
                    text: JSON.stringify(metaData)
                });
            }

            localStorage.reviewQuestions = JSON.stringify(this.reviewQuestions);
        },
        saveReviewBundle: function(){
            var that = this;
            if(this.reviewBundle && this.reviewBundle.length>0) {
                quizAPI.callURL("saveReviewQuestion", {
                    userID: app.getUserID(),
                    questions: this.reviewBundle
                }, function(){
                    that.reviewBundle = [];
                });
            }
        },
        deleteQuestionFromReview: function(questions){
            var that = this;

            questions.forEach(function(questionId){
                questionsWithId = app.reviewQuestions.filter(function(reviewQuestion){
                    return questionId == reviewQuestion.questionId;
                });

                if(questionsWithId.length > 0) {
                    that.reviewQuestions.removeItem(questionsWithId[0]);
                }
            });

            localStorage.reviewQuestions = JSON.stringify(this.reviewQuestions);
            app.player.pause();

            quizAPI.callURL("deleteReviewQuestions", {
                userID: app.getUserID(),
                questionIds: questions
            });
        },
        renderReviewPage: function(field, userScope){
            app.addPage(function(){
                app.renderReviewPage();
            });

            app.appBody.addClass("playing");

            var that = this;
            app.clearBody();

            this.currentPage = 0;
            this.loadReviewQuestions();

            this.element = dom(".review-page", [
                dom(".game-header-wrapper", [
                    // dom(".arrow.left.back-button.blue"),
                    this.header = dom("header", [
                        this.renderLogo(),
                        this.categoryLabel = dom(".active-category")
                    ])
                ]),
                this.questionBody = dom(".question-body"),
                this.answers = dom(".answers", [
                    this.prevButton = dom(".prev.arrow.left"),
                    this.okButton = dom(".ok-i-understand"),
                    this.nextButton = dom(".next.arrow")
                ])
            ]);

            this.prevButton.addEventListener("click", this.prevQuestion.bind(this));
            this.prevButton.addEventListener("touchend", this.prevQuestion.bind(this));

            this.nextButton.addEventListener("click", this.nextQuestion.bind(this));
            this.nextButton.addEventListener("touchend", this.nextQuestion.bind(this));


            this.okButton.addEventListener("click", this.removeQuestion.bind(this));
            this.okButton.addEventListener("touchend", this.removeQuestion.bind(this));

            app.appBody.appendChild(this.element);
            this.renderQuestion();

            this.initSwipe();

            this.recalculateQuestionHeight();

            window.addEventListener('resize', this.recalculateQuestionHeight.bind(this));
            window.addEventListener('orientationchange', this.recalculateQuestionHeight.bind(this));
        },
        initSwipe: function(){
            var startX,
                startY,
                dist,
                threshold = 50, //required min distance traveled to be considered swipe
                allowedTime = 500, // maximum time allowed to travel that distance
                elapsedTime,
                startTime;

            this.element.addEventListener('touchstart', function(e){
                var touchobj = e.changedTouches[0];

                dist = 0;
                startX = touchobj.pageX;
                startY = touchobj.pageY;
                startTime = new Date().getTime(); // record time when finger first makes contact with surface
                e.preventDefault();
            });

            this.element.addEventListener('touchmove', function(e){
                e.preventDefault() // prevent scrolling when inside DIV
            });

            this.element.addEventListener('touchend', function(e){
                var touchobj = e.changedTouches[0], swipeRight, swipeLeft;

                dist = touchobj.pageX - startX;// get total dist traveled by finger while in contact with surface
                elapsedTime = new Date().getTime() - startTime; // get time elapsed
                // check that elapsed time is within specified, horizontal dist traveled >= threshold, and vertical dist traveled <= 100
                swipeRight = (elapsedTime <= allowedTime && dist >= threshold && Math.abs(touchobj.pageY - startY) <= 100);
                swipeLeft = (elapsedTime <= allowedTime && -dist >= threshold && Math.abs(touchobj.pageY - startY) <= 100);
                swipeLeft && app.nextQuestion();
                swipeRight && app.prevQuestion();
                e.preventDefault();
            });
        },
        renderQuestion: function(){
            while(this.currentPage >= this.reviewQuestions.length) {
                this.currentPage--;
            }

            var question = this.reviewQuestions[this.currentPage],
                metaData = question.text ? JSON.parse(question.text) : {},
                that = this;

            function render(question, otherQuestion){
                var questionDataType = question.questionType,
                    answerDataType = question.answerType,
                    value = question.question,
                    correctAnswer = question.answer,
                    wrongAnswer = metaData.wrongAnswer;

                that.questionElement = that["render" + questionDataType](value, question, "question");

                if(metaData.wrongAnswer) {
                    that.answerElement = dom(".item.multi", [
                        that["render" + answerDataType](wrongAnswer, otherQuestion, "answer"),
                        that["render" + answerDataType](correctAnswer, question, "answer")
                    ]);
                } else {
                    that.answerElement = dom(".item.multi", [
                        dom(".item.number.red", _("Too bad! You chose NO but it was the correct answer.")),
                        that["render" + answerDataType](correctAnswer, question, "answer")
                    ]);
                }

                if(answerDataType === "Sound") {
                    app.player.playQuestion(question, "answer");
                }

                that.questionBody.innerHTML = "";

                that.questionBody.appendChild(
                    dom(".container.question-container", [
                        that.questionElement,
                        that.answerElement,
                        dom("span.answer-animation.fa.fa-check"),
                        dom("span.answer-animation.fa.fa-times")
                    ])
                );
            }

            db.questionStore.get(question.questionId, function(question){
                if(metaData.otherQuestionId) {
                    db.questionStore.get(metaData.otherQuestionId, function(otherQuestion){
                        render(question, otherQuestion);
                    });
                } else {
                    render(question);
                }
            });

            this.categoryLabel.innerHTML = metaData.categoryName;

            if(metaData.categoryName.length > 13) {
                this.categoryLabel.addClass("long-name");
            }

            this.showNavigation();
        },
        renderColor: function(value){
            return dom(".color.item", {style: "background: " + value + ";"});
        },
        renderText: function(value){
            return dom(".item.number", value);
        },
        renderPhoto: function(photo, question, prefix){
            var that = this,
                prefix = prefix || "question",
                element =  dom(".item.image", [
                    this.image = dom(".img")
                ]);

            app.storage.get(question[prefix + "_image_crc"], photo["type"], function(imageSrc){
                that.image.setAttribute("style", "background: url(" + imageSrc + ") center center no-repeat; background-size: contain;");
            });

            return element;
        },
        renderSound: function(sound, question, prefix){
            var that = this,
                playButton,
                prefix = prefix || "question",
                element;

            element = dom(".item.sound", {}, [
                playButton = dom("i.fa.fa-volume-up")
            ]);

            element.addEventListener(('ontouchstart' in window)?"touchstart":"click", function(){
                app.player.playQuestion(question, prefix);
            });



            return element;
        },
        nextQuestion: function() {
            if(this.currentPage < this.reviewQuestions.length) {
                this.currentPage++;
                this.renderQuestion();
            }
        },
        prevQuestion: function() {
            if(this.currentPage > 0) {
                this.currentPage--;
                this.renderQuestion();
            }
        },
        removeQuestion: function(){
            var question = this.reviewQuestions[this.currentPage];
            this.deleteQuestionFromReview([question.questionId]);

            this.reviewQuestions.removeItem(question);

            if(this.reviewQuestions.length > 0) {
                this.renderQuestion();
            } else {
                app.appBody.removeClass("playing");
                app.renderMenuPage();
            }
        },
        showNavigation: function(){
            if(this.currentPage>0) {
                $(this.prevButton).show();
            } else {
                $(this.prevButton).hide();
            }

            if(this.currentPage < (this.reviewQuestions.length-1)) {
                $(this.nextButton).show();
            } else {
                $(this.nextButton).hide();
            }
        },
        recalculateQuestionHeight: function(){
            var height = $(window).height() - $(this.header).height() - $(this.answers).height();
            $(this.questionBody).height(height);
        },
        isReviewAvailable: function(){
            return this.reviewQuestions && this.reviewQuestions.length > 0;
        }
    }

    window.Review = Review;
})(window);