(function(global){
    function fail(error) {
        console.log(error.code);
    }

    function DefaultContentProcessor(){}

    DefaultContentProcessor.prototype = {
        encode: function(content){ return content },
        decode: function(content){ return content }
    }

    function IosContentProcessor(){}

    IosContentProcessor.prototype = {
        encode: function(content){ return base32k.encodeBytes(content); },
        decode: function(content){ return base32k.decodeBytes(content); }
    }

    function CordovaStorage(){
        var that = this;

        window.requestFileSystem(LocalFileSystem.PERSISTENT, 0, function (fileSystem) {
            window.resolveLocalFileSystemURL(cordova.file.dataDirectory, function(root){
                that.root = root;
            }, fail);
        });
    }

    CordovaStorage.prototype = {
        store: function(fileData, callback){
            var fileContent = Base64Binary.decodeArrayBuffer(fileData.content),
                fileName = [fileData.crc, fileData.fileType].join(".").replace(/\//g, "-");

            this.root.getFile(fileName, {create: true, exclusive: false}, function(fileEntry){
                fileEntry.createWriter(function(fileWritter){
                    fileWritter.onwriteend = function(){ callback(); }
                    fileWritter.write(fileContent);
                }, fail);
            }, fail);
        },
        get: function (crc, fileType, callback) {
            var fileName = [crc, fileType].join(".").replace(/\//g, "-");
            this.root.getFile(fileName, null, function(fileEntry) {
                callback(fileEntry.toURL())
            }, function(){console.log(arguments)})
        },
        load: function(){},
        save: function(){},
        hasKey: function(key){ return true; }
    }

    function DelayedStorage(){
        var that = this;
        this.storageName = "assets-content-storage";
        this.data = {};

        if(navigator.userAgent.match(/(iPod|iPhone|iPad)/) && !window.cordova) {
            setTimeout(function(){
                that.load();
            }, 1000);
        } else {
            this.load();
        }

        if(navigator.userAgent.match(/(iPod|iPhone|iPad)/)) {
            this.contentProcessor = new IosContentProcessor();
        } else {
            this.contentProcessor = new DefaultContentProcessor();
        }
    }

    DelayedStorage.prototype = {
        store: function(fileData, callback){
            this.data[fileData.crc + "." + fileData.fileType] = this.contentProcessor.encode(fileData.content);
            callback && callback();
        },
        get: function(crc, fileType, callback){
            var that = this,
                prefixes = {
                    "mp3": "data:audio/mp3;base64,",
                    "ogg": "data:audio/ogg;base64,",
                    "jpg": "data:image/jpeg;base64,",
                    "jpeg": "data:image/jpeg;base64,",
                    "png": "data:image/png;base64,",
                    "gif": "data:image/gif;base64,",
                },
                key = crc + "." + fileType;

            callback(
                prefixes[fileType] + this.contentProcessor.decode(this.data[key])
            );
        },
        hasKey: function(key){ return key in this.data },
        save: function(){
            var that = this, i, portion = {}, portionCount = 0, count = 0, countInPortion = 20;

            for(i in this.data) if(this.data.hasOwnProperty(i)) {
                portion[i] = this.data[i];

                count++;
                if(count >= countInPortion) {

                    this.savePortion(this.storageName + "-" + portionCount, portion);
                    portion = {};
                    portionCount++;
                    count = 0;
                }
            }

            this.savePortion(this.storageName + "-" + portionCount, portion);
            localStorage.assetsPortions = portionCount;
        },
        savePortion: function(portionName, portion){
            var that = this;
            localforage.setItem(portionName, JSON.stringify(portion), function(item, e){
                if(e){
                    console.log(e);
                    if(e.name === 'QuotaExceededError' || e.name === 'NS_ERROR_DOM_QUOTA_REACHED' || e.code === e.QUOTA_ERR){
                        console.log(e);
                        setTimeout(function(){
                            that.savePortion(portionName, portion);
                        }, 1000);
                    }
                }
            });
        },
        load: function(){
            var that = this, portionCount = +localStorage.assetsPortions, i=0;

            for(i=0;i<=portionCount;i++) {
                localforage.getItem(this.storageName + "-" + i, function(data){
                    var key;
                    if(data){
                        data = JSON.parse(data);
                        for(key in data) {
                            that.data[key] = data[key];
                        }
                    } else {
                        console.log("empty")
                    }
                });
            }
        }
    }

    function getStorage(){
        if(window.cordova) {
            return new CordovaStorage();
        } else {
            return new DelayedStorage();
        }
    }

    global.getStorage = getStorage;

})(window);