(function(global){
    // 'use strict';

    // Mixin with page layouts
    function UserViews(){}

    UserViews.prototype = {
        renderStartPage: function(){
            app.prevPages = [];
            app.addPage(function(){
                app.user.renderStartPage();
            });

            var that = this;

            app.clearBody();

            var page = dom(".page.page-without-back-button.user-start-page", [
                dom("header", [
                    app.renderLogo(),
                    dom(".the-first-contest-wrapper", [
                        dom(".the-first-contest")
                    ]),
                    dom(".yes-no-game.sign-painter", "yes no game"),
                ]),
                dom(".content", [
                    dom(".home-start-page-wrapper", [
                        this.playButton = dom(".button.play-on-start-page", _("Play"))
                    ]),
                    dom(".buttons", [
                        this.loginButton = dom(".button", _("Login")),
                        this.registerButton = dom(".button", _("Register")),
                        this.playOnFacebookButton = dom(".play-on-facebook-container", [
                            dom(".play-on-facebook-header", {}, [
                                dom("span", _("Register with Facebook")),
                                dom("span.fa.fa-facebook-square")
                            ]),
                            dom(".play-on-facebook-footer", {}, [
                                dom("span", _("Win 50 flags")),
                                dom(".go.arrow.blue")
                            ])
                        ])
                    ]),
                    dom("a.privacy-link", {
                        innerHTML: _("Privacy policy"),
                        target: "_blank",
                        href: "privacypolicy.htm"
                    })
                ])
            ]);

            this.appBody.appendChild(page);
            if(!app.doNotTrackHistory) {
                window.history.pushState({
                    page: "renderStartPage",
                    scope: "user"
                }, "Yes No Game");
            } else {
                app.doNotTrackHistory = false;
            }

            this.playButton.addEventListener("click", function(){
                that.playAnonymously();
            });

            this.playOnFacebookButton.addEventListener("click", function(){
                that.signInWithFacebook();
            });

            this.loginButton.addEventListener("click", this.renderSignInPage.bind(this));
            this.registerButton.addEventListener("click", this.renderRegisterPage.bind(this));

            app.hideCordovaSplashScreen();
        },
        renderSignInPage: function(){
            app.addPage(function(){
                app.user.renderSignInPage();
            });

            var that = this;
            app.clearBody();

            var page = dom(".page", [
                dom("header", [
                    app.renderLogo(),
                    dom(".the-first-contest-wrapper", [
                        dom(".the-first-contest")
                    ])
                ]),
                dom("form", [
                    dom(".form-wrapper", [
                        dom("header", _("Identification with email address")),
                        dom(".fields", [
                            this.errorsArea = dom(".errors"),
                            dom(".label", _("Please enter your email address")),
                            this.emailInput = dom("input#email", {placeholder: _("Email address")}),
                            dom(".label", _("Password")),
                            this.passwordInput = dom("input#password", {"type": "password", placeholder: "aabbcc123"})
                        ]),
                        dom(".fields-button-wrapper", [
                            this.playButton = dom(".play.button", _("Play"))
                        ]),
                        this.forgotPasswordButton = dom(".forgot-button", _("Forgot your password?"))
                    ])
                ])
            ]);

            this.appBody.appendChild(page);
            if(!app.doNotTrackHistory) {
                window.history.pushState({
                    page: "renderSignInPage",
                    scope: "user"
                }, "Yes No Game");
            } else {
                app.doNotTrackHistory = false;
            }

            this.playButton.addEventListener("click", function(e){ that.signIn(); });
            this.forgotPasswordButton.addEventListener("click", this.renderForgotPasswordPage.bind(this));
        },
        renderRegisterPage: function(e, anonymousID){
            app.addPage(function(){
                app.user.renderSignInPage(e, anonymousID);
            });

            var that = this;

            app.clearBody();

            var page = dom(".page", [
                dom("header", [
                    app.renderLogo(),
                    dom(".the-first-contest-wrapper", [
                        dom(".the-first-contest")
                    ])
                ]),
                this.form = dom("form", [
                    dom(".form-wrapper", [
                        dom("header", _("Registration with email address")),
                        dom(".fields", [
                            this.errorsArea = dom(".errors"),
                            dom(".label", _("Please enter your email address")),
                            this.emailInput = dom("input#email", {placeholder: _("Email address")}),
                            dom(".label", "Help your friends to find you"),
                            this.usernameInput = dom("input#username", {placeholder: _("User name")}),
                            dom(".label", _("Birthday")),
                            this.birthdayInput = dom("input#birthday", {
                                placeholder: "dd/mm/yyyy"
                            }),
                            dom(".label", _("Password")),
                            this.passwordInput = dom("input#password", {placeholder: "aabbcc123", autocapitalize: "off"}),
                            this.anonymousIDInput = dom("input", { "type": "hidden", value: anonymousID||"" })
                        ]),
                        dom(".fields-button-wrapper", [
                            this.registerButton = dom(".button", _("Play"))
                        ])
                    ])
                ])
            ]);

            new Formatter(this.birthdayInput, {
                'pattern': '{{99}}/{{99}}/{{9999}}',
            });

            this.appBody.appendChild(page);
            if(!app.doNotTrackHistory) {
                window.history.pushState({
                    page: "renderRegisterPage",
                    scope: "user"
                }, "Yes No Game");
            } else {
                app.doNotTrackHistory = false;
            }

            this.form.addClass("fadeInUpBig");

            this.registerButton.addEventListener("click", this.register.bind(this));
        },
        renderLinkFacebookPage: function(email, facebookID){
            app.addPage(function(){
                app.user.renderSignInPage(email, facebookID);
            });

            var that = this;

            app.clearBody();

            var page = dom(".page", [
                dom("header", [
                    app.renderLogo(),
                    dom(".the-first-contest-wrapper", [
                        dom(".the-first-contest")
                    ])
                ]),
                dom("form", [
                    dom(".form-wrapper", [
                        dom("header", _("Link facebook account to email")),
                        dom(".fields", [
                            this.errorsArea = dom(".errors"),
                            dom(".label", _("Please enter your email address")),
                            this.emailInput = dom("input#email", {placeholder: _("Email address"), value: email}),
                            dom(".label", _("Password")),
                            this.passwordInput = dom("input#password", {placeholder: "aabbcc123"}),
                        ]),
                        dom(".fields-button-wrapper", [
                            this.linkButton = dom(".button", _("Link"))
                        ])
                    ])
                ])
            ]);

            this.appBody.appendChild(page);

            this.linkButton.addEventListener("click", function(){
                that.signIn(that.emailInput.value, that.passwordInput.value, function(user){
                    that.linkFacebook(user.userID, facebookID, app.renderMenuPage.bind(app));
                });
            });
        },
        renderForgotPasswordPage: function(){
            app.addPage(function(){
                app.user.renderForgotPasswordPage();
            });

            var that = this;
            app.clearBody();

            var page = dom(".forgot-password-page.page", [
                dom("header", [
                    app.renderLogo(),
                    dom(".the-first-contest-wrapper", [
                        dom(".the-first-contest")
                    ])
                ]),
                dom("form", [
                    dom(".form-wrapper", [
                        dom("header", _("Forgot your password?")),
                        dom(".fields", [
                            this.errorsArea = dom(".errors"),
                            dom(".label", _("Please enter your email address")),
                            this.emailInput = dom("input#email", {placeholder: _("Email address")})
                        ]),
                        dom(".fields-button-wrapper", [
                            this.sendCodeButton = dom(".button", _("Send")),
                        ])
                    ])
                ])
            ]);

            this.appBody.appendChild(page);
            if(!app.doNotTrackHistory) {
                window.history.pushState({
                    page: "renderForgotPasswordPage",
                    scope: "user"
                }, "Yes No Game");
            } else {
                app.doNotTrackHistory = false;
            }

            this.sendCodeButton.addEventListener("click", function(){
                that.requestRestorePassword();
            });
        },
        renderRestorePasswordPage: function(){
            app.addPage(function(){
                app.user.renderRestorePasswordPage();
            });

            var that = this;
            app.clearBody();

            var page = dom(".restore-password-page.page", [
                dom("header", [
                    app.renderLogo(),
                    dom(".the-first-contest-wrapper", [
                        dom(".the-first-contest")
                    ])
                ]),
                dom("form", [
                    dom(".form-wrapper", [
                        dom("header", _("Restore your password")),
                        dom(".fields", [
                            this.errorsArea = dom(".errors"),
                            dom(".label", _("Please enter your email address")),
                            this.emailInput = dom("input#email", {placeholder: _("Email address")}),
                            dom(".label", _("Please enter the code that you received")),
                            this.codeInput = dom("input#code", {placeholder: "Code"}),
                            dom(".label", _("Password")),
                            this.passwordInput = dom("input#password", {placeholder: "aabbcc123"}),
                        ]),
                        dom(".fields-button-wrapper", [
                            this.restorePasswordButton = dom(".button", _("Restore password"))
                        ])
                    ])
                ])
            ]);

            this.appBody.appendChild(page);
            if(!app.doNotTrackHistory) {
                window.history.pushState({
                    page: "renderRestorePasswordPage",
                    scope: "user"
                }, "Yes No Game");
            } else {
                app.doNotTrackHistory = false;
            }

            this.restorePasswordButton.addEventListener("click", function(){
                that.restorePassword();
            });
        },
        renderProfilePage: function(){
            app.addPage(function(){
                app.user.renderProfilePage();
            });

            var that = this;
            app.clearBody();

            var page = dom(".profile-page.page", [
                dom("header", [
                    app.renderLogo(),
                    dom(".the-first-contest-wrapper", [
                        dom(".the-first-contest")
                    ])
                ]),
                dom(".content", [
                    dom("form", [
                        dom("header", _("My account")),
                        this.formFields = dom(".fields", [
                            this.errorsArea = dom(".errors"),
                            dom(".label", _("Email address")),
                            this.emailInput = dom("input#email", {placeholder: "sarahdupond@gmail.com", value: app.user.user.email}),
                            dom(".label", _("User name")),
                            this.usernameInput = dom("input#username", {placeholder: "sarah_flower", value: app.user.user.username}),
                            dom(".label", _("Birthday")),
                            this.birthdayInput = dom("input#birthday", {placeholder: "dd/mm/yyyy", value: app.user.user.birthday||"", autocomplete: "off"})
                        ])
                    ]),
                    dom("form", [
                        dom("header", _("Password")),
                        dom(".fields", [
                            this.passErrorsArea = dom(".errors"),
                            this.passwordInput = dom("input#password", {"type": "password", placeholder: _("New password")}),
                        ])
                    ]),
                    dom(".buttons", [
                        this.saveAndCloseButton = dom(".button", _("Save and close")),
                        this.logoutButton = dom(".button", _("Logout"))
                    ])
                ])
            ]);

            if(app.isSchoolRankingEnabled()) {
                this.renderSchoolSelector(this.formFields);
            }

            new Formatter(this.birthdayInput, {
                'pattern': '{{99}}/{{99}}/{{9999}}',
            });

            this.appBody.appendChild(page);
            if(!app.doNotTrackHistory) {
                window.history.pushState({
                    page: "renderProfilePage",
                    scope: "user"
                }, "Yes No Game");
            } else {
                app.doNotTrackHistory = false;
            }

            this.saveAndCloseButton.addEventListener("click", this.saveUserProfile.bind(this));

            this.logoutButton.addEventListener("click", this.logout.bind(this));
        },
        renderSchoolSelector: function(wrapper){
            var that = this;
            this.schoolFieldsWrapper = dom(".school-selection-wrapper", [
                dom(".label", _("Select country")),
                this.countrySelect = dom("select#country.country-sel", [
                    dom("option", { value: "-1", innerHTML: _("Loading ...") })
                ]),
                dom(".label", _("Select region")),
                this.regionSelect = dom("select#region.region-sel", [
                    dom("option", { value: "-1", innerHTML: _("Loading ...") })
                ]),
                dom(".label", _("Select city")),
                this.citySelect = dom("select#city,city-sel", [
                    dom("option", { value: "-1", innerHTML: _("Loading ...") })
                ]),
                dom(".label", _("Select school")),
                this.schoolSelect = dom("select#school.school-sel", [
                    dom("option", { value: "-1", innerHTML: _("Loading ...") })
                ])
            ]);

            var country = app.user.user.countryCode,
                region, city, school;

            if(app.user.user.schoolId) {
                quizAPI.callURL("locationBySchoolId", {
                    schoolId: app.user.user.schoolId
                }, function(data){
                    country = data.data.countryId;
                    region = data.data.regionId
                    city = data.data.city
                    school = data.data.id;

                    loadRegions(country);
                    loadCities(region);
                    loadSchools(region, city);
                });
            }

            function countryChange(e){
                var val = $(e.target).val();
                if (val === country || !val) return;
                region = city = school = false;
                country = val;
                loadRegions(country);
            }

            function loadRegions(country){
                quizAPI.callURL("regions", {
                    countryId: country
                }, function(data){
                    that.regionSelect.innerHTML = "";
                    if(data.regions.length>0) {
                        var option = dom("option", { value: "-1", innerHTML: _("No region") });
                        that.regionSelect.appendChild(option);

                        data.regions.forEach(function(regionObject){
                            var option = dom("option", { value: regionObject.id, innerHTML: regionObject.name });

                            if(regionObject.id===region) {
                                option.setAttribute("selected", "selected");
                            }

                            that.regionSelect.appendChild(option);
                        });
                    } else {
                        var option = dom("option", { value: "-1", innerHTML: _("No regions found") });
                        that.regionSelect.appendChild(option);
                    }

                    setDisableStatus();
                }, function(){
                    that.regionSelect.innerHTML = "";

                    var option = dom("option", { value: "-1", innerHTML: "No regions found" });
                    that.regionSelect.appendChild(option);
                    $(that.regionSelect).prop("selectedIndex", -1);
                    setDisableStatus();
                });
            }

            function regionChange(e){
                var val = $(e.target).val();
                if (val === region || !val) return;
                city = school = false;
                region = val;
                loadCities(region);
            }

            function loadCities(region){
                quizAPI.callURL("cities", {
                    regionId: region
                }, function(data){
                    that.citySelect.innerHTML = "";
                    if(data.cities.length>0) {
                        var option = dom("option", { value: "-1", innerHTML: _("No city") });
                        that.citySelect.appendChild(option);

                        data.cities.forEach(function(cityObject){
                            var option = dom("option", { value: cityObject.id, innerHTML: cityObject.name });

                            if(cityObject.id===city) {
                                $(option).prop("selected", true);
                            }

                            that.citySelect.appendChild(option);
                        });
                    } else {
                        var option = dom("option", { value: "-1", innerHTML: _("No cities found") });
                        that.citySelect.appendChild(option);
                    }

                    setDisableStatus();
                }, function(){
                    that.citySelect.innerHTML = "";

                    var option = dom("option", { value: "-1", innerHTML: "No cities found" });
                    that.citySelect.appendChild(option);
                    $(that.citySelect).prop("selectedIndex", -1);
                    setDisableStatus();
                });
            }

            function cityChange(e){
                var val = $(e.target).val();
                if (val === school || !val) return;
                school = false;
                city = val;
                loadSchools(region, city);
            }

            function loadSchools(region, city){
                quizAPI.callURL("schools", {
                    regionId: region,
                    cityId: city
                }, function(data){
                    that.schoolSelect.innerHTML = "";

                    if(data.schools.length > 0) {
                        var option = dom("option", { value: "-1", innerHTML: _("No school") });
                        that.schoolSelect.appendChild(option);

                        data.schools.forEach(function(schoolObject){
                            var option = dom("option", { value: schoolObject.id, innerHTML: schoolObject.name });

                            if(schoolObject.id===+school) {
                                $(option).prop("selected", true);
                            }

                            that.schoolSelect.appendChild(option);
                        });
                    } else {
                        var option = dom("option", { value: "-1", innerHTML: _("No schools found") });
                        that.schoolSelect.appendChild(option);
                    }

                    setDisableStatus();
                }, function(){
                    that.schoolSelect.innerHTML = "";

                    var option = dom("option", { value: "-1", innerHTML: "No schools found" });
                    that.schoolSelect.appendChild(option);
                    $(that.schoolSelect).prop("selectedIndex", -1);
                    setDisableStatus();
                });
            }

            function setDisableStatus() {
                $(that.regionSelect).prop("disabled", !country);
                $(that.citySelect).prop("disabled", !region);
                $(that.schoolSelect).prop("disabled", !city);

                if(!region) {
                    $(that.citySelect).prop("selectedIndex", -1);
                }

                if(!city) {
                    $(that.schoolSelect).prop("selectedIndex", -1);
                }
            }

            $(this.countrySelect).on("change", countryChange);
            $(this.regionSelect).on("change", regionChange);
            $(this.citySelect).on("change", cityChange);

            wrapper.appendChild(this.schoolFieldsWrapper);

            quizAPI.callURL("countries", {}, function(data){
                that.countrySelect.innerHTML = "";

                data.countries.forEach(function(country){
                    var option = dom("option", { value: country.id, innerHTML: country.name });

                    if(country.id===app.user.user.countryCode) {
                        option.setAttribute("selected", "selected");
                        loadRegions(app.user.user.countryCode);
                    }

                    that.countrySelect.appendChild(option);
                });
            });
        },
        renderInviteByEmailPage: function(){
            app.addPage(function(){
                app.user.renderInviteByEmailPage();
            });

            var that = this;

            this.emailInputs = [];

            app.clearBody();

            var page = dom(".page.invite-friends-page", [
                dom("header", [
                    app.renderLogo(),
                    dom(".the-first-contest-wrapper", [
                        dom(".the-first-contest")
                    ])
                ]),
                dom("form", [
                    dom(".form-wrapper", [
                        dom("header", _("Invite friends")),
                        dom(".fields", [
                            this.errorsArea = dom(".errors"),
                            dom(".label", "Please enter your friends&prime; email addresses"),
                            this.emailInputs[0] = dom("input", {placeholder: _("Email address")}),
                            this.emailInputs[1] = dom("input", {placeholder: _("Email address")}),
                            this.emailInputs[2] = dom("input", {placeholder: _("Email address")}),
                            this.emailInputs[3] = dom("input", {placeholder: _("Email address")}),
                            this.emailInputs[4] = dom("input", {placeholder: _("Email address")})
                        ]),
                        dom(".fields-button-wrapper", [
                            this.inviteButtons = dom(".button", _("Invite"))
                        ])
                    ])
                ])
            ]);

            this.appBody.appendChild(page);
            if(!app.doNotTrackHistory) {
                window.history.pushState({
                    page: "renderInviteByEmailPage",
                    scope: "user"
                }, "Yes No Game");
            } else {
                app.doNotTrackHistory = false;
            }

            this.inviteButtons.addEventListener("click", function(){
                var emails = that.emailInputs.map(function(input){
                    return input.value;
                }).filter(function(value){
                    return value;
                });

                quizAPI.callURL("inviteFriendsByEmail", {
                    url: settings.inviteURL,
                    userID: that.user.userID,
                    emails: emails
                }, function(result){
                    app.renderMenuPage();

                }, that.validate.bind(that))
            });
        },
        renderJoinPage: function(e, noThanksCallback){
            if(!app.joinPageWasRendered){
                app.addPage(function(){
                    app.user.renderJoinPage(e, noThanksCallback);
                });
                app.joinPageWasRendered = true;
            }

            var that = this;

            app.clearBody();

            var page = dom(".page.user-start-page.join-page", [
                dom("header", [
                    app.renderLogo(),
                    dom(".the-first-contest-wrapper", [
                        dom(".the-first-contest")
                    ])
                ]),
                this.contentElement = dom(".content", [
                    dom(".features-descriptions", [
                        dom(".feature-title", _("As soon as you open your account:")),
                        dom(".feature", _("The game adapts to your level")),
                        dom(".feature", _("You accumulate your points")),
                        dom(".feature", _("You save your categories")),
                        dom(".feature", _("You access all the rankings")),
                    ]),
                    dom(".home-start-page-wrapper", [
                        this.playOnFacebookButton = dom(".play-on-facebook-container", [
                            dom(".play-on-facebook-header", {}, [
                                dom("span", _("Register with Facebook")),
                                dom("span.fa.fa-facebook-square")
                            ]),
                            dom(".play-on-facebook-footer", {}, [
                                dom("span", _("Win 50 flags")),
                                dom(".go.arrow.blue")
                            ])
                        ])
                    ]),
                    dom(".buttons.join", [
                        this.registerButton = dom(".button.long", _("Register with email")),
                    ])
                ])
            ]);

            if(noThanksCallback) {
                page.addClass("page-without-back-button");
                var noThanksButton;
                this.contentElement.appendChild(
                    dom(".no-thanks", [
                        noThanksButton = dom("span", "3")
                    ])
                );

                setTimeout(function(){
                    noThanksButton.innerHTML = "2";
                }, 1000);

                setTimeout(function(){
                    noThanksButton.innerHTML = "1";
                }, 2000);

                setTimeout(function(){
                    noThanksButton.innerHTML = _("No thanks");
                    noThanksButton.addEventListener("click", noThanksCallback);
                }, 3000);
            }

            this.appBody.appendChild(page);
            this.contentElement.addClass("fadeInUpBig");

            if(!app.doNotTrackHistory) {
                window.history.pushState({
                    page: "renderJoinPage",
                    scope: "user"
                }, "Yes No Game");
            } else {
                app.doNotTrackHistory = false;
            }

            this.playOnFacebookButton.addEventListener("click", function(){
                app.user.signInWithFacebook(app.renderMenuPage.bind(app), app.getUserID());
            });

            this.registerButton.addEventListener("click", function(e){
                app.user.renderRegisterPage(e, app.getUserID());
            });
        },
        renderFeedbackPage: function(){
            app.addPage(function(){
                app.user.renderFeedbackPage();
            });

            var that = this;

            app.clearBody();

            var page = dom(".page.feedback-page", [
                dom("header", [
                    app.renderLogo(),
                    dom(".the-first-contest-wrapper", [
                        dom(".the-first-contest")
                    ])
                ]),
                dom("form", [
                    dom(".form-wrapper", [
                        dom("header", _("Report a problem")),
                        dom(".fields", [
                            this.errorsArea = dom(".errors"),
                            this.textArea = dom("textarea", {placeholder: _("Message")}),
                        ]),
                        dom(".fields-button-wrapper", [
                            this.postFeedbackButton = dom(".button", _("Send"))
                        ])
                    ])
                ])
            ]);

            this.appBody.appendChild(page);
            if(!app.doNotTrackHistory) {
                window.history.pushState({
                    page: "renderFeedbackPage",
                    scope: "user"
                }, "Yes No Game");
            } else {
                app.doNotTrackHistory = false;
            }

            this.postFeedbackButton.addEventListener("click", function(){
                var message = that.textArea.value,
                    data = {
                        message: message,
                        userID: that.user.userID,
                        screen_resolution: parseInt($(window).width()) + "x" + parseInt($(window).height()),
                        os: navigator.platform,
                        ua: navigator.userAgent
                    };

                if(window.cordova) {
                    data.appVersion = window.AppVersion;
                }

                quizAPI.callURL("postFeedback", data, function(result){
                    app.renderMenuPage();
                }, that.validate.bind(that))
            });
        }
    }

    window.UserViews = UserViews;
})(window);