(function(global){
    'use strict';

    function User(){
        this.appBody = document.getElementById("appBody");
    }

    User.prototype = {
        validate: function(XMLHttpRequest, textStatus, errorThrown){
            this.errorsArea.innerText = XMLHttpRequest.responseJSON.message;
        },
        playAnonymously: function(callback){
            var callback = callback || app.renderMenuPage.bind(app),
                that = this,
                anonymousUser, userData,
                onSuccess, onFailure;

            onSuccess = function(result){
                localStorage.anonymousUser = JSON.stringify({
                    email: userData.email,
                    password: userData.password,
                    userID: result.user.id
                });
                that.findOrCreateUser(result, callback);
            }

            onFailure = function(result){
                localStorage.removeItem("anonymousUser");
                that.playAnonymously(callback);
            }

            if(localStorage.anonymousUser) {
                anonymousUser = JSON.parse(localStorage.anonymousUser)
            }

            if(anonymousUser && anonymousUser !== -1) {
                userData = JSON.parse(localStorage.anonymousUser);
                quizAPI.callURL("signin", userData, onSuccess, onFailure);
            } else {
                var timestamp = +(new Date()),
                    username = "anonym" + timestamp,
                    email = "anonym" + timestamp + "@gmail.com",
                    birthday = "19/10/1987",
                    password = "anonym" + timestamp;

                userData = {
                    email: email,
                    username: username,
                    password: password,
                    birthday: birthday,
                    accountType: "anonymous"
                }
                quizAPI.callURL("register", userData, onSuccess, onFailure);
            }
        },
        signIn: function(email, password, callback){
            var that=this;
            callback = callback || app.renderMenuPage.bind(app);

            quizAPI.callURL("signin", {
                email: email || this.emailInput.value,
                password: password || this.passwordInput.value

            }, function(result){
                app.signedInWithFacebook = false;
                that.findOrCreateUser(result, callback);
            }, this.validate.bind(this))
        },
        signInWithSSO: function(token, callback, fail_callback){
            var that=this;
            callback = callback || app.renderMenuPage.bind(app);

            quizAPI.callURL("signinWithSSO", {
                token: token
            }, function(result){
                app.signedInWithFacebook = false;
                that.findOrCreateUser(result, callback);
            }, fail_callback);
        },
        register: function(){
            var that = this,
                callback = app.renderMenuPage.bind(app);

            quizAPI.callURL("register", {
                email: this.emailInput.value,
                username: this.usernameInput.value,
                password: this.passwordInput.value,
                birthday: this.birthdayInput.value||"19/10/1987",
                anonymousId: this.anonymousIDInput.value,
                accountType: "default"
            }, function(result){
                delete localStorage.anonymousUser;
                that.findOrCreateUser(result, callback);
            }, this.validate.bind(this))
        },
        logout: function(){
            this.user = null;
            delete localStorage.currentUserID;
            delete localStorage.categoriesUnlocked;
            delete localStorage.notPassedCategories;
            app.categoriesUnlocked = [];
            app.notPassedCategories = [];

            this.renderStartPage();
        },
        requestRestorePassword: function(){
            quizAPI.callURL("requestRestorePassword", {
                email: this.emailInput.value
            }, this.renderRestorePasswordPage.bind(this), this.validate.bind(this));
        },
        restorePassword: function(){
            quizAPI.callURL("restorePassword",{
                email: this.emailInput.value,
                code: this.codeInput.value,
                password: this.passwordInput.value
            }, this.renderSignInPage.bind(this), this.validate.bind(this));
        },
        loadScore: function(callback){
            var that = this;

            quizAPI.callURL("loadScore", {
                userID: this.user.userID
            }, function(result){
                var score = result.score,
                    categoriesUnlocked = result.categoriesUnlocked,
                    notPassedCategories = result.notPassedCategories;

                localStorage.offlineUnlockedCategories = JSON.stringify([]);

                for(var scoreOption in score) if(score.hasOwnProperty(scoreOption)) {
                    that.user[scoreOption] = score[scoreOption];
                }

                app.categoriesUnlocked = [];
                app.notPassedCategories = [];

                categoriesUnlocked.forEach(function(category){
                    app.categoriesUnlocked.push(+category.id);
                });

                notPassedCategories.forEach(function(categoryInfo){
                    app.notPassedCategories.push(+categoryInfo.id);
                    db.categoryStore.update(categoryInfo.id, {
                        pointsToAvailable: categoryInfo.goalUnlock
                    });
                })

                localStorage.categoriesUnlocked = JSON.stringify(app.categoriesUnlocked);
                localStorage.notPassedCategories = JSON.stringify(app.notPassedCategories);

                db.userStore.put(that.user, function(){
                    callback && callback();
                });

            }, function(XMLHttpRequest, textStatus, errorThrown){
                console.log(XMLHttpRequest.responseJSON.message);
            })
        },
        loadRanking: function(pointScope, userScope, timeScope, callback){
            var that = this,
                apiMap = {
                    "points": {
                        "last1000": "last1000Ranking",
                        "curMonth": "monthRanking",
                        "prevMonth": "monthRanking",
                        "prevPrevMonth": "monthRanking"
                    },
                    "categories": {
                        "last1000": "categoryRanking",
                        "curMonth": "categoryRanking",
                        "prevMonth": "categoryRanking",
                        "prevPrevMonth": "categoryRanking"
                    }
                },
                month = {
                    "curMonth": "cur",
                    "prevMonth": "prev",
                    "prevPrevMonth": "prevPrev"
                },
                apiMethod = apiMap[pointScope][timeScope];

            quizAPI.callURL(apiMethod, {
                userID: this.user.userID,
                month: month[timeScope],
                limit: 300
            }, function(result){
                if(!(pointScope in that.user)) {
                    that.user[pointScope] = {};
                }

                if(!(timeScope in that.user[pointScope])) {
                    that.user[pointScope][timeScope] = {};
                }

                that.user[pointScope][timeScope].myRanking = result.user;
                that.user[pointScope][timeScope].global = result.global;
                that.user[pointScope][timeScope].friends = result.friends;
                that.user[pointScope][timeScope].school = result.school;
                that.user[pointScope][timeScope].country = result.country;

                db.userStore.put(that.user, function(){
                    callback && callback();
                });

            }, function(XMLHttpRequest, textStatus, errorThrown){
                console.log(XMLHttpRequest.responseJSON.message);
            })
        },
        signInWithFacebook: function(callback, anonymousId){
            var that = this, meSuccessCallback, loginSuccessCallback;
            callback = callback || app.renderMenuPage.bind(app);

            loginSuccessCallback = function(response){
                if (response.authResponse) {
                    var facebookID = response.authResponse.userID;

                    quizAPI.callURL("signinWithFacebook", {
                        facebookId: facebookID
                    }, function(result){
                        app.signedInWithFacebook = true;
                        that.putFacebookFriends(result.user.id);
                        that.findOrCreateUser(result, callback);
                    }, function(XMLHttpRequest, textStatus, errorThrown){
                        if(XMLHttpRequest.responseJSON.message==="Facebook ID not found") {
                            that.registerWithFacebook(callback, anonymousId);
                        }
                    });
                } else {
                    that.renderStartPage();
                }
            }

            if(window.cordova) {
                FB.login(['email', 'user_birthday', 'user_friends'], loginSuccessCallback);
            } else {
                FB.login(loginSuccessCallback, {scope: "email,user_birthday,user_friends"});
                // FB.api("/me", meSuccessCallback);
            }
        },
        registerWithFacebook: function(callback, anonymousId){
            var that = this;
            FB.api("/me", window.cordova?[]: {scope: "email,user_birthday,user_friends"}, function(response){
                var userBirthday = moment(response.birthday||response.user_birthday, "MM/DD/YYYY").format("DD/MM/YYYY");
                quizAPI.callURL("registerWithFacebook", {
                    email: response.email,
                    username: response.name||response.username||response.email.replace(/@.*$/,""),
                    facebookId: response.id,
                    birthday: userBirthday,
                    anonymousId: anonymousId||""
                }, function(result){
                    that.putFacebookFriends(result.user.id);
                    that.findOrCreateUser(result, callback);
                }, function(XMLHttpRequest, textStatus, errorThrown){
                    if(XMLHttpRequest.responseJSON.message === "Email already exists") {
                        that.renderLinkFacebookPage(response.email, response.id);
                    }
                });
            });
        },
        putFacebookFriends: function(userID){
            FB.api("/me/friends", window.cordova?[]: {scope: "email,user_birthday,user_friends"},
                function (response) {
                    var friends = response.data.map(function(friend){
                        return friend.id;
                    });

                    quizAPI.callURL("putFacebookFriends", {
                        userID: userID,
                        friends: friends
                    });
                }
            );
        },
        inviteFacebookFriends: function(){
            var userID = app.getUserID();

            if(window.cordova) {
                FB.showDialog({
                    method: "apprequests",
                    message: _("Play Quiz game with me!")
                }, function(response){
                    // Object {to[0]: "100003237919657", request: "1489147084666653", to[1]: "100002008386568"}
                    var friends = [], friendCount = 0;

                    while(("to[" + friendCount + "]") in response) {
                        friends.push(response["to[" + friendCount + "]"]);
                        friendCount++;
                    }

                    quizAPI.callURL("inviteFacebookFriends", {
                        userID: userID,
                        friends: friends,
                        request: response.request
                    });

                }, function(){
                    console.log(arguments)
                });
            } else {
                FB.ui({
                    method: 'apprequests',
                    message: _("Play Yes No Game with me!")
                }, function(response){
                    if(!response.to) return;

                    var friends = response.to.map(function(friendId){
                        return friendId;
                    });

                    quizAPI.callURL("inviteFacebookFriends", {
                        userID: userID,
                        friends: friends,
                        request: response.request
                    });
                });
            }
        },
        linkFacebook: function(userID, facebookID, callback){
            quizAPI.callURL("linkFacebook", {
                userID: userID,
                facebookId: facebookID
            }, function(){
                that.putFacebookFriends(userID);
                callback();
            }, this.validate.bind(this));
        },
        findOrCreateUser: function(data, callback){
            var that = this,
                userInfo = data.user;

            localStorage.currentUserID = userInfo.id;

            db.userStore.put(
                {
                    userID: userInfo.id,
                    email: userInfo.email,
                    username: userInfo.username,
                    birthday: userInfo.birthday,
                    facebookId: userInfo.facebookId,
                    countryCode: userInfo.registerCountryCode,
                    country: userInfo.registerCountry,
                    schoolAge: userInfo.schoolAge,
                    schoolId: userInfo.schoolId,
                    schoolName: userInfo.schoolName
                },
                function(dbUser){
                    db.userStore.get(
                        userInfo.id,
                        function(dbUser){
                            that.user = dbUser;
                            that.registerDevice();
                            that.showAds();
                            app.syncReviewQuestions(function(){
                                app.storage.save();
                            });
                            that.loadScore();
                            callback(that.user);
                        },
                        function(){
                            setTimeout(function(){
                                that.findOrCreateUser(data, callback);
                            }, 50);
                        }
                    )
                }
            );
        },
        showAds: function(){
            var prohibitCountries = ["FR", "DE", "ES", "AT", "SE", "PL", "DK", "NO", "FI", "NL"],
                allowForCountry = (prohibitCountries.indexOf(app.user.user.countryCode) === -1);

            if(allowForCountry && app.isDesktop() && $(window).width() > 1000) {
                window.showAds();
                document.body.addClass("adsense");
            }
        },
        usePrevTimeUser: function(onSuccess, onFail){
            var that = this;
            if(localStorage.currentUserID && +localStorage.currentUserID != -1) {
                db.userStore.get(localStorage.currentUserID, function(dbUser){
                    that.user = dbUser;
                    that.registerDevice();
                    that.showAds();
                    app.syncReviewQuestions(function(){
                        app.storage.save();
                    });
                    if(!that.user.country) {
                        that.updateCountryIfDoNoExist(onSuccess);
                    } else {
                        onSuccess();
                    }
                }, function(){
                    onFail();
                });
            } else {
                onFail();
            }
        },
        updateCountryIfDoNoExist: function(callback){
            var that = this,
                data = {
                    userID: this.user.userID,
                    email: this.user.email,
                    username: this.user.username,
                    birthday: this.user.birthday
                };

            quizAPI.callURL("updateProfile", data, function(result){
                var user = result.user;

                that.user.country = user.registerCountry;

                db.userStore.put(that.user, function(){
                    callback && callback();
                });

            });
        },
        registerDevice: function(){
            if(window.cordova && !this.user.isDeviceRegistered) {
                registerDevice(this.user);
                this.user.isDeviceRegistered = true;
                db.userStore.put(this.user);
            }
        },
        useAnonymousUserOffline: function(callback){
            var that = this,
                anonymousUser = null;

            if(localStorage.anonymousUser) {
                anonymousUser = JSON.parse(localStorage.anonymousUser);
            }

            if(anonymousUser && anonymousUser.userID) {
                db.userStore.get(+anonymousUser.userID, function(dbUser){
                    that.user = dbUser;
                    callback();
                });
            } else {
                var timestamp = +(new Date()),
                    username = "anonym" + timestamp,
                    email = "anonym" + timestamp + "@gmail.com",
                    birthday = "19/10/1987",
                    password = "anonym" + timestamp;

                anonymousUser = {
                    userID: -1,
                    username: username,
                    email: email,
                    birthday: birthday,
                    password: password
                }

                that.user = anonymousUser;
                callback();
            }
        },
        saveUserProfile: function(callback){
            var that = this;
            if(!app.onLine) {
                loading.start();
                app.checkOnline(false, this.saveUserProfile.bind(this), function(){
                    loading.stop();
                    that.errorsArea.innerHTML = _("We can't connect to Yes No Game");
                });
                return;
            }

            var data = {
                userID: that.user.userID,
                email: that.emailInput.value,
                username: that.usernameInput.value,
                birthday: that.birthdayInput.value
            }

            if(that.schoolSelect) {
                data.schoolId = $(that.schoolSelect).val() || "-1";
            }

            if(that.passwordInput.value) {
                if(that.passwordInput.value.length < 4) {
                    that.passErrorsArea.innerHTML = _("Your password must be at least 4 characters length");
                    return;
                }
                data.password = that.passwordInput.value;
            } else {
                that.passErrorsArea.innerHTML = "";
            }

            quizAPI.callURL("updateProfile", data, function(result){
                var user = result.user;

                that.user.email = user.email;
                that.user.birthday = user.birthday;
                that.user.username = user.username;
                that.user.schoolId = user.schoolId;
                that.user.schoolName = user.schoolName;
                that.user.countryCode = user.registerCountryCode;
                that.user.country = user.registerCountry;

                db.userStore.put(that.user, function(){
                    app.renderMenuPage();
                });

            }, that.validate.bind(that));
        },
        saveSchool: function(callback){
            var that = this;
            if(!app.onLine) {
                loading.start();
                app.checkOnline(false, this.saveSchool.bind(this), function(){
                    loading.stop();
                    that.errorsArea.innerHTML = _("We can't connect to Yes No Game");
                });
                return;
            }

            var data = {
                userID: that.user.userID,
                email: that.user.email,
                username: that.user.username,
                birthday: that.user.birthday,
                schoolId: ($(this.schoolSelect).val() || "-1")
            }

            quizAPI.callURL("updateProfile", data, function(result){
                var user = result.user;

                that.user.schoolId = user.schoolId;
                that.user.schoolName = user.schoolName;
                that.user.countryCode = user.registerCountryCode;
                that.user.country = user.registerCountry;

                db.userStore.put(that.user, function(){
                    callback && callback();
                });

            }, that.validate.bind(that))
        },
        isSignIn: function(){
            return !!this.user;
        },
        isFacebookUser: function(){
            return !!this.user.facebookId;
        }
    }

    augment(User, UserViews);
    augment(User, UserScore);
    // augment(User, UserSchools);

    window.User = User;
})(window);